import moment from 'moment';

import BD from '../connection.js';
import ExceptionHandler from '../Exceptions/ExceptionHandler.js';
import PermissionManager from './PermissionManager.js';
import NovoCodigo from '../Providers/NovoCodigo.js';

// TODO Review BackOfficeExceptionHandler
const BackOfficeException = Error;

const CardapioInteligente = {
    requiredParams: [
        'CDFILIAL',  'CDSERVFILI',  'DTREFCUSPADR', 'IDTIPORESTRICAO',
        'DTMESCARD', 'NRTIPOCUSTO', 'DTREFECUSTO',  'DTMES',
        'DIAS_CONG'
    ],
    async testConnection(){
        const result = await this.execQuery(`SELECT TO_CHAR(SYSDATE, 'DD/MM/YYYY HH24:MI:SS') FROM DUAL`);
        return result;
    },
    validateParameters( params ){
        return this.requiredParams.every( requiredParam =>
            params.includes(requiredParam)
        );
    },
    async isValidServiceLink( NRGERACAOCARD, NRITGERACAOCARD ){
        const SQL = `SELECT IT.CDFILIAL, IT.CDSERVFILI, IT.NRCENARIOVINCULO, TPCMS.CDFILIALORIG, TPCMS.CDSERVORIG, CASE WHEN TPCMS.CDFILIALORIG IS NOT NULL AND IT.NRCENARIOVINCULO IS NULL THEN 'N' ELSE 'S' END VALID_ITEM FROM ( SELECT CDFILIAL, CDSERVFILI, NRCENARIOVINCULO FROM ITGERACAOCARD WHERE NRGERACAOCARD   = :P_NRGERACAOCARD AND NRITGERACAOCARD = :P_NRITGERACAOCARD ) IT LEFT JOIN TIPOPRATCOMUMSERV TPCMS ON IT.CDFILIAL   = TPCMS.CDFILIAL AND IT.CDSERVFILI = TPCMS.CDSERVFILI`;
        const result = await this.execQuery(SQL, {
            P_NRGERACAOCARD: NRGERACAOCARD,
            P_NRITGERACAOCARD: NRITGERACAOCARD
        });
        const [ FIRST_RECORD = { VALID_ITEM: 'N' } ] = result.status == 'SUCCESS' ? result.data : [];
        const {
            VALID_ITEM
        } = FIRST_RECORD;
        return VALID_ITEM == 'S';
    },
    async execQuery(SQL, params){
        let status = "SUCCESS";
        let data = [];
        try {
            data = await BD.execQuery(SQL, params);
        } catch ( error ){
            data = ExceptionHandler.getErrorMessage(error);
            status = ExceptionHandler.statusHandler(error);
        }
        return {
            status,
            data
        };
    },
    async execProc(name, params){
        let status = "SUCCESS";
        let data = [];
        try {
            data = await BD.execProc(name, params);
        } catch ( error ){
            data = ExceptionHandler.getErrorMessage(error);
            status = ExceptionHandler.statusHandler(error);
        }
        return { status, data };
    },
    async getConfiguracaoServico( params ){
        const SQL = `SELECT CF.CDFILIAL AS COD_FILIAL, CF.CDSERVFILI AS COD_SERVICO, CF.NRSEQCNFCARD AS NUM_SEQUENCIA, CF.CDTIPOPRAT AS COD_TIP_PRATO, TP.NMTIPOPRAT || CASE WHEN CF.DSCOMPLTP IS NULL THEN '' ELSE '(' || CF.DSCOMPLTP || ') ' END AS NMTIPOPRAT, CF.IDOPCCNFCARD AS IND_OPCAO, CG.NRGRUPOPRATOSERV AS NUM_GRP_PRATO, TP.CDGRTIPOPR AS COD_GRP_TIP_PRATO, GR.NMGRTIPOPR AS NOME_GRP_TIP_PRATO, CF.IDUTILPERCENTCONS AS ATRATIVIDADE, CP.VRCUSTPADR, NVL(PP.VRPEMINDIFPLAPA, 0) AS PERC_CUSTO_MIN, NVL(PP.VRPEMAXDIFPLAPA, 0) AS PERC_CUSTO_MAX, DECODE(NVL(PP.VRPEMINDIFPLAPA, 0), 0, CP.VRCUSTPADR, CP.VRCUSTPADR - CP.VRCUSTPADR * PP.VRPEMINDIFPLAPA / 100) AS VLR_CUSTO_MIN, DECODE(NVL(PP.VRPEMAXDIFPLAPA, 0), 0, CP.VRCUSTPADR, CP.VRCUSTPADR + CP.VRCUSTPADR * PP.VRPEMAXDIFPLAPA / 100) AS VRL_CUSTO_MAX, CF.IDOTMCOLUNAFIXA, CF.IDOTMNAOCONSCOR, CF.IDOTMNAOCONSMOD, CF.IDOTMINTERCPRAT FROM CONFCARD CF, CUSTPADR CP, CONFCARDGRUPOPRATO CG, GRUPOPRATOSERV GS, GRUPOPRATO GP, TIPOPRAT TP, PARAMPLANEJA PP, GRTIPOPR GR, CONFCARDMESATIVO CFA WHERE CF.CDFILIAL = :P_CDFILIAL AND CF.CDSERVFILI = :P_CDSERVFILI AND CP.CDFILIAL(+) = CF.CDFILIAL AND CP.CDSERVFILI(+) = CF.CDSERVFILI AND CP.DTREFCUSPADR(+) = :P_DTREFCUSPADR AND CG.CDFILIAL (+) = CF.CDFILIAL AND CG.CDSERVFILI (+) = CF.CDSERVFILI AND CG.NRSEQCNFCARD (+) = CF.NRSEQCNFCARD AND CF.CDTIPOPRAT = TP.CDTIPOPRAT AND PP.CDFILIAL = CF.CDFILIAL AND GR.CDGRTIPOPR = TP.CDGRTIPOPR AND GS.NRGRUPOPRATOSERV = CG.NRGRUPOPRATOSERV AND GS.NRGRUPOPRATO = GP.NRGRUPOPRATO AND GP.IDTIPORESTRICAO = :P_IDTIPORESTRICAO AND GS.CDSERVFILI = CG.CDSERVFILI AND GP.CDFILIAL = CG.CDFILIAL AND CFA.CDFILIAL(+) = CF.CDFILIAL AND CFA.CDSERVFILI(+) = CF.CDSERVFILI AND CFA.NRSEQCNFCARD(+) = CF.NRSEQCNFCARD AND SUBSTR(NVL(CFA.IDMESATIVO,'SSSSSSSSSSSS'), TO_NUMBER(TO_CHAR(TO_DATE(:P_DTREFCUSPADR), 'MM')),1) = 'S'`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTREFCUSPADR:     {val: params.DTREFCUSPADR},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO},
            P_DTREFCUSPADR:     {val: params.DTREFCUSPADR}
        });
    },
    async getFrequenciaGrupoPratoMes( params ){
        const SQL = `SELECT V.CDFILIAL AS COD_FILIAL, F.CDSERVFILI AS COD_SERVICO, TO_CHAR(V.DTVIGENCIA, 'MM/YYYY')|| CASE WHEN V.DESCRICAO IS NULL THEN '' ELSE ' - ' || V.DESCRICAO END AS VIGENCIA, AG.NMAGRUPGRPRAT AS NOME_AGRUPAMENTO, G.NMGRUPOPRATO AS NOME_GRUPO, G.NMGRUPOPRATO AS NOME_GRUPO_PRATO, IT.NRGRUPOPRATOSERV AS NUM_GRP_PRATO, IT.NRMINFREQGRPRATMES AS MIN_MES_GRP, IT.NRMAXFREQGRPRATMES AS MAX_MES_GRP, IT.NRMAXFREQGRPRATSEM AS MAX_SEM_GRP FROM VIGFREQGRPRATMES V INNER JOIN FREQGRPRATMES F ON F.NRVIGENCIA = V.NRVIGENCIA INNER JOIN ITFREQGRPRATMES IT ON IT.NRFREQGRPRATMES = F.NRFREQGRPRATMES INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = IT.NRGRUPOPRATOSERV INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = GS.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO = :P_IDTIPORESTRICAO WHERE V.CDFILIAL = :P_CDFILIAL AND F.CDSERVFILI = :P_CDSERVFILI AND V.IDTIPORESTRICAO = :P_IDTIPORESTRICAO AND V.NRVIGENCIA =(SELECT NRVIGENCIA FROM (SELECT VV.NRVIGENCIA, VV.DTVIGENCIA, CASE WHEN ( VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM >= :P_DTMESCARD AND VV.DTVIGENCIA <= :P_DTMESCARD) THEN 0 WHEN ( VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM <= :P_DTMESCARD ) THEN 2 WHEN VV.DTVIGENCIA <= :P_DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGFREQGRPRATMES VV WHERE :P_CDFILIAL = VV.CDFILIAL AND VV.IDTIPORESTRICAO = :P_IDTIPORESTRICAO ORDER BY ORDEM, DTVIGENCIA DESC ) WHERE ROWNUM = 1) ORDER BY AG.NMAGRUPGRPRAT, G.CDGRUPOPRATO`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMESCARD:        {val: params.DTMESCARD},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO}
        });
    },
    async getFrequenciaOpcoesGrupoPrato ( params ) {
        const SQL = `SELECT V.CDFILIAL AS COD_FILIAL, F.CDSERVFILI AS COD_SERVICO, IT.NRGRUPOPRATOSERV AS NUM_GRP_PRATO, IT.NRMINFREQOPCAGR AS MIN_MES_OPC, IT.NRMAXFREQOPCAGR AS MAX_MES_OPC, IT.NRMAXFREQOPCAGRSEM AS MAX_SEM_OPC FROM VIGFREQOPCAGR V INNER JOIN FREQOPCAGR F ON F.NRVIGENCIA = V.NRVIGENCIA INNER JOIN ITFREQOPCAGR IT ON IT.NRFREQOPCAGR = F.NRFREQOPCAGR WHERE V.CDFILIAL =:P_CDFILIAL AND F.CDSERVFILI =:P_CDSERVFILI AND V.IDTIPORESTRICAO =:P_IDTIPORESTRICAO AND V.NRVIGENCIA =(SELECT NRVIGENCIA FROM (SELECT VV.NRVIGENCIA, VV.DTVIGENCIA, CASE WHEN ( VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM >= :P_DTMESCARD AND VV.DTVIGENCIA <= :P_DTMESCARD) THEN 0 WHEN ( VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM <= :P_DTMESCARD ) THEN 2 WHEN VV.DTVIGENCIA <= :P_DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGFREQOPCAGR VV WHERE :P_CDFILIAL = VV.CDFILIAL AND VV.IDTIPORESTRICAO = :P_IDTIPORESTRICAO ORDER BY ORDEM, DTVIGENCIA DESC ) WHERE ROWNUM = 1)`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMESCARD:        {val: params.DTMESCARD},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO}
        });
    },
    async getFrequenciaPratosMes ( params ) {
        const SQL = `SELECT DISTINCT V.CDFILIAL AS COD_FILIAL, F.CDSERVFILI AS COD_SERVICO, TO_CHAR( V.DTVIGENCIA, 'MM/YYYY') || CASE WHEN V.DESCRICAO IS NULL THEN '' ELSE ' - ' || V.DESCRICAO END AS VIGENCIA, CASE WHEN P.CDFILAUXPRAT = 'P' THEN 'Padrão' ELSE 'Local' END AS ARVORE, P.CDPRATO AS CODIGO_PRATO, P.NMPRATO NOME_PRATO, IT.CDFILAUXPRAT || IT.CDPRATO AS COD_PRATO, IT.NRMINFREQPRATMES AS MIN_MES_PRATO, IT.NRMAXFREQPRATMES AS MAX_MES_PRATO, IT.NRMAXFREQPRATSEM AS MAX_SEMANA, CASE WHEN PT.CDPRATO IS NULL THEN 'N' ELSE 'S' END AS POSSUI_GRUPO FROM VIGFREQPRATMES V INNER JOIN FREQPRATMES F ON F.NRVIGENCIA = V.NRVIGENCIA INNER JOIN ITFREQPRATMES IT ON IT.NRFREQPRATMES = F.NRFREQPRATMES INNER JOIN PRATO P ON P.CDFILAUXPRAT = IT.CDFILAUXPRAT AND P.CDPRATO = IT.CDPRATO LEFT JOIN( SELECT ITG.CDFILAUXPRAT, ITG.CDPRATO FROM ITGRUPOPRATOSERV ITG INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = ITG.NRGRUPOPRATOSERV AND GS.CDSERVFILI =:P_CDSERVFILI INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.CDFILIAL =:P_CDFILIAL AND G.IDTIPORESTRICAO =:P_IDTIPORESTRICAO ) PT ON PT.CDFILAUXPRAT = P.CDFILAUXPRAT AND PT.CDPRATO = P.CDPRATO WHERE V.CDFILIAL =:P_CDFILIAL AND F.CDSERVFILI =:P_CDSERVFILI AND V.IDTIPORESTRICAO =:P_IDTIPORESTRICAO AND V.NRVIGENCIA = (SELECT NRVIGENCIA FROM (SELECT VV.NRVIGENCIA, VV.DTVIGENCIA, CASE WHEN ( VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM >= :P_DTMESCARD AND VV.DTVIGENCIA <= :P_DTMESCARD ) THEN 0 WHEN ( VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM <= :P_DTMESCARD ) THEN 2 WHEN VV.DTVIGENCIA <= :P_DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGFREQPRATMES VV WHERE :P_CDFILIAL = VV.CDFILIAL AND VV.IDTIPORESTRICAO = :P_IDTIPORESTRICAO ORDER BY ORDEM, DTVIGENCIA DESC ) WHERE ROWNUM = 1) ORDER BY POSSUI_GRUPO, P.CDPRATO, P.NMPRATO`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMESCARD:        {val: params.DTMESCARD},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO}
        });
    },
    async getFrequenciaProdutosGrupoPrato ( params ) {
        const SQL = `SELECT V.CDFILIAL AS COD_FILIAL, F.CDSERVFILI AS COD_SERVICO, TO_CHAR( V.DTVIGENCIA, 'MM/YYYY') || CASE WHEN V.DESCRICAO IS NULL THEN '' ELSE ' - ' || V.DESCRICAO END AS VIGENCIA, AG.NMAGRUPGRPRAT AS NOME_AGRUPAMENTO, G.NMGRUPOPRATO AS NOME_GRUPO, G.NMGRUPOPRATO AS NOME_GRUPO_PRATO, IT.NRGRUPOPRATOSERV, IT.CDPRODUTO AS COD_PRODUTO, P.CDARVPROD AS ARVORE_PRODUTO, P.NMPRODUTO AS NOME_PRODUTO, IT.NRMINPRODGRPRAT AS MIN_MES_PRD, IT.NRMAXPRODGRPRAT AS MAX_MES_PRD, IT.NRMAXPRODGRPRATSEM AS MAX_SEMANA FROM VIGFREQPRODGRPRAT V INNER JOIN FREQPRODGRPRAT F ON F.NRVIGENCIA = V.NRVIGENCIA INNER JOIN ITFREQPRODGRPRAT IT ON IT.NRFREQPRODGRPRAT = F.NRFREQPRODGRPRAT INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = IT.NRGRUPOPRATOSERV INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.IDTIPORESTRICAO =:P_IDTIPORESTRICAO LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = GS.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO =:P_IDTIPORESTRICAO INNER JOIN PRODUTO P ON P.CDPRODUTO = IT.CDPRODUTO WHERE V.CDFILIAL =:P_CDFILIAL AND F.CDSERVFILI =:P_CDSERVFILI AND V.IDTIPORESTRICAO =:P_IDTIPORESTRICAO AND V.NRVIGENCIA =(SELECT NRVIGENCIA FROM (SELECT VV.NRVIGENCIA, VV.DTVIGENCIA, CASE WHEN ( VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM >= :P_DTMESCARD AND VV.DTVIGENCIA <= :P_DTMESCARD ) THEN 0 WHEN ( VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM <= :P_DTMESCARD ) THEN 2 WHEN VV.DTVIGENCIA <= :P_DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGFREQPRODGRPRAT VV WHERE :P_CDFILIAL = VV.CDFILIAL AND VV.IDTIPORESTRICAO = :P_IDTIPORESTRICAO ORDER BY ORDEM, DTVIGENCIA DESC ) WHERE ROWNUM = 1) ORDER BY AG.NMAGRUPGRPRAT, G.NMGRUPOPRATO, P.CDARVPROD`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMESCARD:        {val: params.DTMESCARD},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO}
        });
    },
    async getGrupoPrato ( params ) {
        const SQL = `SELECT COD_FILIAL, COD_SERVICO, CDAGRUPGRPRAT, NMAGRUPGRPRAT, CDGRUPOPRATO, NOM_GRUPOPRATO, NUM_GRP_PRATO, COD_PRATO, NMPRATO, CASE WHEN VLR_CUSTO_PRATO <= 0 THEN NULL ELSE VLR_CUSTO_PRATO END AS VLR_CUSTO_PRATO, CASE WHEN VLR_CUSTO_PRATO_4 <= 0 THEN NULL ELSE VLR_CUSTO_PRATO_4 END AS VLR_CUSTO_PRATO_4 FROM ( SELECT G.CDFILIAL AS COD_FILIAL, GS.CDSERVFILI AS COD_SERVICO, AG.CDAGRUPGRPRAT, AG.NMAGRUPGRPRAT, G.CDGRUPOPRATO, G.NMGRUPOPRATO AS NOM_GRUPOPRATO, GS.NRGRUPOPRATOSERV AS NUM_GRP_PRATO, IT.CDFILAUXPRAT || IT.CDPRATO AS COD_PRATO, P.NMPRATO, CUSTO_PRATO(:P_CDFILIAL, :P_CDFILIAL, :P_NRTIPOCUSTO, :P_DTREFECUSTO, IT.CDFILAUXPRAT, IT.CDPRATO, :P_CDSERVFILI, 'S', NULL, NULL, -3) VLR_CUSTO_PRATO, CUSTO_PRATO(:P_CDFILIAL, :P_CDFILIAL, :P_NRTIPOCUSTO, ADD_MONTHS(TO_DATE(:P_DTREFECUSTO, 'DD/MM/YYYY'),-4), IT.CDFILAUXPRAT, IT.CDPRATO, :P_CDSERVFILI, 'S', NULL, NULL, 0) VLR_CUSTO_PRATO_4, ( SELECT DISTINCT 'S' AS IDTIPO FROM RECESERV LEFT JOIN PRATSERV ON PRATSERV.CDPRATO = RECESERV.CDPRATO AND PRATSERV.CDFILAUXPRAT = RECESERV.CDFILAUXPRAT AND PRATSERV.CDFILIAL = RECESERV.CDFILIAL AND PRATSERV.CDSERVFILI = RECESERV.CDSERVFILI WHERE ( RECESERV.CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( RECESERV.CDPRATO = P.CDPRATO ) AND ( RECESERV.CDFILIAL = :P_CDFILIAL ) AND ( RECESERV.CDSERVFILI = :P_CDSERVFILI ) AND ( NVL(PRATSERV.IDATIVO, 'S') = 'S' ) UNION ALL SELECT DISTINCT 'L' AS IDTIPO FROM RECEFILI LEFT JOIN PRATFILI ON PRATFILI.CDPRATO = RECEFILI.CDPRATO AND PRATFILI.CDFILAUXPRAT = RECEFILI.CDFILAUXPRAT AND PRATFILI.CDFILIAL = RECEFILI.CDFILIAL WHERE ( RECEFILI.CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( RECEFILI.CDPRATO = P.CDPRATO ) AND ( RECEFILI.CDFILIAL = :P_CDFILIAL ) AND ( NVL(PRATFILI.IDATIVO, 'S') = 'S' ) AND NOT EXISTS ( SELECT CDPRATO FROM RECESERV WHERE ( CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( CDPRATO = P.CDPRATO ) AND ( CDFILIAL = :P_CDFILIAL ) AND ( CDSERVFILI = :P_CDSERVFILI ) ) UNION ALL SELECT DISTINCT 'P' AS IDTIPO FROM RECEPADR LEFT JOIN PRATPADR ON PRATPADR.CDPRATO = RECEPADR.CDPRATO AND PRATPADR.CDFILAUXPRAT = RECEPADR.CDFILAUXPRAT WHERE ( RECEPADR.CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( RECEPADR.CDPRATO = P.CDPRATO ) AND ( NVL(PRATPADR.IDATIVO, 'S') = 'S' ) AND NOT EXISTS ( SELECT CDPRATO FROM RECEFILI WHERE ( CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( CDPRATO = P.CDPRATO ) AND ( CDFILIAL = :P_CDFILIAL ) ) AND NOT EXISTS ( SELECT CDPRATO FROM RECESERV WHERE ( CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( CDPRATO = P.CDPRATO ) AND ( CDFILIAL = :P_CDFILIAL ) AND ( CDSERVFILI = :P_CDSERVFILI ) ) ) AS TIPORECEITA FROM GRUPOPRATO G INNER JOIN GRUPOPRATOSERV   GS ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO INNER JOIN ITGRUPOPRATOSERV IT ON IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV INNER JOIN PRATO P ON P.CDFILAUXPRAT = IT.CDFILAUXPRAT AND P.CDPRATO = IT.CDPRATO LEFT JOIN AGRUPGRPRAT      AG ON GS.NRAGRUPGRPRAT = AG.NRAGRUPGRPRAT WHERE G.CDFILIAL = :P_CDFILIAL AND GS.CDSERVFILI = :P_CDSERVFILI AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO AND P.IDATIVO = 'S' ) P CROSS JOIN PARAMGERAL PG WHERE ( ( PG.IDUTRPADCA = 'N' AND P.TIPORECEITA <> 'P' ) OR ( PG.IDUTRPADCA = 'S' ) ) AND P.TIPORECEITA IS NOT NULL ORDER BY CASE WHEN VLR_CUSTO_PRATO IS NULL THEN VLR_CUSTO_PRATO ELSE 0 END, P.CDAGRUPGRPRAT, P.CDGRUPOPRATO, P.COD_PRATO`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_NRTIPOCUSTO:      {val: params.NRTIPOCUSTO},
            P_DTREFECUSTO:      {val: params.DTREFECUSTO},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO}
        });
    },
    async getGrupoTipoPrato ( params ) {
        const SQL = `SELECT CDGRTIPOPR AS COD_GRP_TIP_PRATO, NMGRTIPOPR AS NOM_GRP_TIP_PRATO, 40 AS PERC_GRP_TIP_PRATO FROM GRTIPOPR`;
        return this.execQuery( SQL, {});
    },
    async getLimitePerCapita ( params ) {
        const SQL = `SELECT V.CDFILIAL AS COD_FILIAL, L.CDSERVFILI AS COD_SERVICO, TO_CHAR( V.DTVIGENCIA, 'MM/YYYY') || CASE WHEN V.DESCRICAO IS NULL THEN '' ELSE ' - ' || V.DESCRICAO END AS VIGENCIA, AG.NMAGRUPGRPRAT AS NOME_AGRUPAMENTO, G.NMGRUPOPRATO AS NOME_GRUPO, IT.NRGRUPOPRATOSERV AS NUM_GRP_PRATO, IT.CDPRODUTO AS COD_PRODUTO, P.CDARVPROD AS ARVORE_PRODUTO, P.NMPRODUTO AS NOME_PRODUTO, TO_CHAR( IT.QTMINLIMPERSERGR, '9999990.99999' ) AS QTD_MIN_GRAMAGEM, TO_CHAR( IT.QTMAXLIMPERSERGR, '9999990.99999' ) AS QTD_MAX_GRAMAGEM FROM VIGLIMPESERGR V INNER JOIN LIMPESERGR L ON L.NRVIGENCIA = V.NRVIGENCIA INNER JOIN ITLIMPESERGR IT ON IT.NRLIMPESERGR = L.NRLIMPESERGR INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = IT.NRGRUPOPRATOSERV INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.IDTIPORESTRICAO =:P_IDTIPORESTRICAO LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = GS.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO =:P_IDTIPORESTRICAO INNER JOIN PRODUTO P ON P.CDPRODUTO = IT.CDPRODUTO WHERE V.CDFILIAL =:P_CDFILIAL AND L.CDSERVFILI =:P_CDSERVFILI AND V.IDTIPORESTRICAO =:P_IDTIPORESTRICAO AND V.NRVIGENCIA =(SELECT NRVIGENCIA FROM (SELECT VV.NRVIGENCIA, VV.DTVIGENCIA, CASE WHEN ( VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM >= :P_DTMESCARD AND VV.DTVIGENCIA <= :P_DTMESCARD ) THEN 0 WHEN ( VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM <= :P_DTMESCARD ) THEN 2 WHEN VV.DTVIGENCIA <= :P_DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGLIMPESERGR VV WHERE :P_CDFILIAL = VV.CDFILIAL AND VV.IDTIPORESTRICAO = :P_IDTIPORESTRICAO ORDER BY ORDEM, DTVIGENCIA DESC ) WHERE ROWNUM = 1) ORDER BY AG.NMAGRUPGRPRAT, G.NMGRUPOPRATO, P.CDARVPROD`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMESCARD:        {val: params.DTMESCARD},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO}
        });
    },
    async getPratoCombinacao ( params ) {
        const SQL = `SELECT COD_FILIAL, COD_SERVICO, COD_GRP_TIP_PRATO, NOME_GRUPO_TIPO_PRATO, TIPO, CASE WHEN TIPO = 'Grupo' THEN DESC_GRUPO_DE_PRATO ELSE DESC_PRATO END AS DESCRICAO, COD_PRATO, NRGRUPOPRATOSERV, COD_GRP_TIP_PRATO_COMB, NOME_GRUPO_TIPO_PRATO_COMB, TIPO_COMB, CASE WHEN TIPO_COMB = 'Grupo' THEN DESC_GRUPO_DE_PRATO_COMB ELSE DESC_PRATO_COMB END AS DESCRICAO_COMB, COD_PRATO_COMB, NRGRUPOPRATOSERV_COMB, SEQ, PERCENTREDUCAO FROM ( SELECT C.CDFILIAL AS COD_FILIAL, C.CDSERVFILI AS COD_SERVICO, C.CDFILAUXPRAT || C.CDPRATO AS COD_PRATO, C.CDGRTIPOPR AS COD_GRP_TIP_PRATO, G1.NMGRTIPOPR AS NOME_GRUPO_TIPO_PRATO, CASE WHEN C.CDFILAUXPRAT IS NOT NULL THEN 'Prato' ELSE 'Grupo' END AS TIPO, CASE WHEN P1.CDFILAUXPRAT = 'P' THEN 'Padrão' ELSE 'Local' END || ' - ' || C.CDPRATO || ' - ' || P1.NMPRATO AS DESC_PRATO, C.NRGRUPOPRATOSERV, GS1.CDGRUPOPRATO || ' - ' || GS1.NMGRUPOPRATO AS DESC_GRUPO_DE_PRATO, I.CDFILAUXPRAT || I.CDPRATO AS COD_PRATO_COMB, I.CDGRTIPOPR AS COD_GRP_TIP_PRATO_COMB, G2.NMGRTIPOPR AS NOME_GRUPO_TIPO_PRATO_COMB, CASE WHEN I.CDFILAUXPRAT IS NOT NULL THEN 'Prato' ELSE 'Grupo' END AS TIPO_COMB, CASE WHEN P2.CDFILAUXPRAT = 'P' THEN 'Padrão' ELSE 'Local' END || ' - ' || I.CDPRATO || ' - ' || P2.NMPRATO AS DESC_PRATO_COMB, I.NRGRUPOPRATOSERV AS NRGRUPOPRATOSERV_COMB, GS2.CDGRUPOPRATO || ' - ' || GS2.NMGRUPOPRATO AS DESC_GRUPO_DE_PRATO_COMB, RTRIM( XMLAGG( XMLELEMENT(E, CC.NRSEQCNFCARD, ';') ).EXTRACT('//text()').GETCLOBVAL(), ',' ) AS SEQ, I.PERCENTREDUCAO FROM COMBGRUPOTPPRAT C INNER JOIN ITCOMBGRUPOTPPRAT I ON I.NRCOMBGRUPOTPPRAT = C.NRCOMBGRUPOTPPRAT LEFT JOIN PRATO P1 ON P1.CDFILAUXPRAT = C.CDFILAUXPRAT AND P1.CDPRATO = C.CDPRATO INNER JOIN GRTIPOPR G1 ON G1.CDGRTIPOPR = C.CDGRTIPOPR LEFT JOIN( SELECT GS.NRGRUPOPRATOSERV, G.CDGRUPOPRATO, G.NMGRUPOPRATO FROM GRUPOPRATOSERV GS INNER JOIN GRUPOPRATO G ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO AND GS.CDSERVFILI =: P_CDSERVFILI AND G.CDFILIAL =: P_CDFILIAL AND G.IDTIPORESTRICAO =: P_IDTIPORESTRICAO ) GS1 ON GS1.NRGRUPOPRATOSERV = C.NRGRUPOPRATOSERV INNER JOIN GRTIPOPR G2 ON G2.CDGRTIPOPR = I.CDGRTIPOPR LEFT JOIN PRATO P2 ON P2.CDFILAUXPRAT = I.CDFILAUXPRAT AND P2.CDPRATO = I.CDPRATO LEFT JOIN ( SELECT GS.NRGRUPOPRATOSERV, G.CDGRUPOPRATO, G.NMGRUPOPRATO FROM GRUPOPRATOSERV GS INNER JOIN GRUPOPRATO G ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO AND GS.CDSERVFILI =: P_CDSERVFILI AND G.CDFILIAL =: P_CDFILIAL AND G.IDTIPORESTRICAO =: P_IDTIPORESTRICAO ) GS2 ON GS2.NRGRUPOPRATOSERV = I.NRGRUPOPRATOSERV LEFT JOIN ( SELECT DISTINCT CC.NRSEQCNFCARD, TP.CDGRTIPOPR FROM CONFCARDGRUPOPRATO CC INNER JOIN CONFCARD CNF ON CC.CDFILIAL = CNF.CDFILIAL AND CC.CDSERVFILI = CNF.CDSERVFILI AND CC.NRSEQCNFCARD = CNF.NRSEQCNFCARD INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = CNF.CDTIPOPRAT WHERE CNF.CDFILIAL =: P_CDFILIAL AND CNF.CDSERVFILI =: P_CDSERVFILI ) CC ON CC.CDGRTIPOPR = I.CDGRTIPOPR WHERE I.IDCOMBINACAO = 'OBRIGATORIO' AND C.CDFILIAL =: P_CDFILIAL AND C.CDSERVFILI =: P_CDSERVFILI GROUP BY C.CDFILIAL, C.CDSERVFILI, C.CDFILAUXPRAT, C.CDPRATO, C.CDGRTIPOPR, G1.NMGRTIPOPR, CASE WHEN C.CDFILAUXPRAT IS NOT NULL THEN 'Prato' ELSE 'Grupo' END, CASE WHEN P1.CDFILAUXPRAT = 'P' THEN 'Padrão' ELSE 'Local' END || ' - ' || C.CDPRATO || ' - ' || P1.NMPRATO, C.NRGRUPOPRATOSERV, GS1.CDGRUPOPRATO, GS1.NMGRUPOPRATO, I.CDFILAUXPRAT, I.CDPRATO, I.CDGRTIPOPR, G2.NMGRTIPOPR, CASE WHEN I.CDFILAUXPRAT IS NOT NULL THEN 'Prato' ELSE 'Grupo' END, CASE WHEN P2.CDFILAUXPRAT = 'P' THEN 'Padrão' ELSE 'Local' END || ' - ' || I.CDPRATO || ' - ' || P2.NMPRATO, I.NRGRUPOPRATOSERV, GS2.CDGRUPOPRATO, GS2.NMGRUPOPRATO, PERCENTREDUCAO ) ORDER BY COD_FILIAL, COD_SERVICO, COD_GRP_TIP_PRATO, TIPO, CASE WHEN TIPO = 'Grupo' THEN DESC_GRUPO_DE_PRATO ELSE DESC_PRATO END, COD_GRP_TIP_PRATO_COMB, TIPO_COMB, CASE WHEN TIPO_COMB = 'Grupo' THEN DESC_GRUPO_DE_PRATO_COMB ELSE DESC_PRATO_COMB END`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO}
        });
    },
    async getPratoEquivalente ( params ) {
        const SQL = `SELECT :P_CDFILIAL AS COD_FILIAL, CDFILAUXPRAT || CDPRATO AS COD_PRATO, CDFILPRATRELAC || CDPRATORELAC AS COD_PRATO_EQUIV FROM PRATORELAC WHERE ( CDFILIAL = :P_CDFILIAL OR CDFILIAL = 'P' )`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL}
        });
    },
    async getReceitaPrato ( params ) {
        const SQL = `SELECT PRA.CDFILIAL AS COD_FILIAL, PRA.CDSERVFILI AS COD_SERVICO, PRA.CDFILAUXPRAT || PRA.CDPRATO AS COD_PRATO, PRA.NMPRATO AS NOM_PRATO, PRA.NRGRUPOPRATOSERV AS NUM_GRP_PRATO, PRA.NMPRODUTO AS NOM_PRODUTO, PRA.CDARVPROD AS COD_ARV_PRODUTO, PRA.CDPRODUTO AS COD_PRODUTO, PRA.QTPRODUTO AS GRAMAGEM, PRA.VRPRATO AS VLR_CUSTO_PRATO, PRA.VRPRATO_4 AS VLR_CUSTO_PRATO_4, PRA.IDCOR AS COR_PRATO, NVL(PRA.IDCOMPLEXIDADE, 'BAIXA') AS COMPLEX_PRATO, NVL(PRA.IDPREPREPARO, 'N') AS PRE_PREPARO, PRA.NRINGREDIENTEPRINC1 AS INGR_PRINCIPAL_1, PRA.NMINGREDIENTEPRINC1 AS NM_INGR_PRINCIPAL_1, PRA.NRINGREDIENTEPRINC2 AS INGR_PRINCIPAL_2, PRA.NMINGREDIENTEPRINC2 AS NM_INGR_PRINCIPAL_2, PRA.CDAPRERECE AS TP_CORTE, PRA.NMAPRERECE AS NM_TP_CORTE, PRA.NRFORMAPREPARO AS MODO_PREPARO, PRA.NMFORMAPREPARO AS NM_MODO_PREPARO, PRA.NRATRATIVIDADEPRATO AS ATRATIVIDADE, PRA.NMATRATIVIDADEPRATO AS NMATRATIVIDADE, PRA.VRPESOATRATIVIDADE FROM ( SELECT :P_CDFILIAL AS CDFILIAL, :P_CDSERVFILI AS CDSERVFILI, PT.CDFILAUXPRAT, PT.CDPRATO, PT.NMPRATO, PRODUTO.NMPRODUTO, PRODUTO.CDARVPROD, REC.CDPRODUTO, REC.QTPRODUTO, PT.IDCOR, PT.IDCOMPLEXIDADE, PT.IDPREPREPARO, PT.VRPRATO, PT.VRPRATO_4, MIN(PRODUTO.IDPRODATIVPR) OVER(PARTITION BY PT.CDFILAUXPRAT, PT.CDPRATO) AS IDPRODATIVPR, MAX(NVL(PRODFILI.IDPRODATIV, 'A')) OVER(PARTITION BY PT.CDFILAUXPRAT, PT.CDPRATO) AS IDPRODATIV, PT.NRINGREDIENTEPRINC1, PT.NMINGREDIENTEPRINC1, PT.NRINGREDIENTEPRINC2, PT.NMINGREDIENTEPRINC2, PT.CDAPRERECE, PT.NMAPRERECE, PT.NRFORMAPREPARO, PT.NMFORMAPREPARO, PT.NRATRATIVIDADEPRATO, PT.NMATRATIVIDADEPRATO, PT.VRPESOATRATIVIDADE, PT.NRGRUPOPRATOSERV FROM ( SELECT PT.CDFILAUXPRAT, PT.CDPRATO, PT.NMPRATO, PTF.IDCOR, PTF.IDCOMPLEXIDADE, PTF.IDPREPREPARO, PRATO_RECEITA_ATIVA(PT.CDPRATO, PT.CDFILAUXPRAT, :P_CDFILIAL, :P_CDSERVFILI) IDATIVO, CUSTO_PRATO(:P_CDFILIAL, :P_CDFILIAL, :P_NRTIPOCUSTO, :P_DTREFECUSTO, PT.CDFILAUXPRAT, PT.CDPRATO, :P_CDSERVFILI, 'S', NULL, NULL, - 3) VRPRATO, CUSTO_PRATO(:P_CDFILIAL, :P_CDFILIAL, :P_NRTIPOCUSTO, ADD_MONTHS(TO_DATE(:P_DTREFECUSTO, 'DD/MM/YYYY'),-4), PT.CDFILAUXPRAT, PT.CDPRATO, :P_CDSERVFILI, 'S', NULL, NULL, 0) VRPRATO_4, PTF.NRINGREDIENTEPRINC1, ING1.NMINGREDIENTEPRINC AS NMINGREDIENTEPRINC1, PTF.NRINGREDIENTEPRINC2, ING2.NMINGREDIENTEPRINC AS NMINGREDIENTEPRINC2, PTF.CDAPRERECE, AP.NMAPRERECE, PTF.NRFORMAPREPARO, FP.NMFORMAPREPARO, PTF.NRATRATIVIDADEPRATO, AT.VRPESOATRATIVIDADE, ITG.NRGRUPOPRATOSERV, AT.NMATRATIVIDADEPRATO FROM PRATO PT INNER JOIN ITGRUPOPRATOSERV ITG ON ITG.CDPRATO = PT.CDPRATO AND ITG.CDFILAUXPRAT = PT.CDFILAUXPRAT INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = ITG.NRGRUPOPRATOSERV AND GS.CDSERVFILI = :P_CDSERVFILI INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.CDFILIAL = :P_CDFILIAL AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO LEFT JOIN PARAMPRATFILIAL PTF ON PT.CDPRATO = PTF.CDPRATO AND PT.CDFILAUXPRAT = PTF.CDFILAUXPRAT AND PTF.CDFILIAL = :P_CDFILIAL LEFT JOIN INGREDIENTEPRINC ING1 ON PTF.NRINGREDIENTEPRINC1 = ING1.NRINGREDIENTEPRINC LEFT JOIN INGREDIENTEPRINC ING2 ON PTF.NRINGREDIENTEPRINC2 = ING2.NRINGREDIENTEPRINC LEFT JOIN ATRATIVIDADEPRATO AT ON AT.NRATRATIVIDADEPRATO = PTF.NRATRATIVIDADEPRATO LEFT JOIN FORMAPREPARO FP ON FP.NRFORMAPREPARO = PTF.NRFORMAPREPARO LEFT JOIN APRERECE AP ON AP.CDAPRERECE = PTF.CDAPRERECE WHERE ( PT.CDFILAUXPRAT = 'P' OR PT.CDFILAUXPRAT = :P_CDFILIAL ) ) PT, TABLE ( RECEITA_UTIL(:P_CDFILIAL, :P_CDSERVFILI, PT.CDPRATO, PT.CDFILAUXPRAT, '0', 'Z') ) REC, PRODFILI, PRODUTO, PARAMGERAL PG WHERE PT.IDATIVO = 'S' AND PT.VRPRATO > 0 AND REC.CDPRATO = PT.CDPRATO AND REC.CDFILAUXPRAT = PT.CDFILAUXPRAT AND PRODUTO.CDPRODUTO = REC.CDPRODUTO AND REC.CDPRODUTO = PRODFILI.CDPRODUTO AND PRODFILI.CDFILIAL = :P_CDFILIAL AND ( ( PG.IDUTRPADCA = 'N' AND REC.IDTIPORECE <> 'P' ) OR ( PG.IDUTRPADCA = 'S' ) ) ) PRA WHERE IDPRODATIVPR = 'S' AND IDPRODATIV = 'A'`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_NRTIPOCUSTO:      {val: params.NRTIPOCUSTO},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO},
            P_DTREFECUSTO:      {val: params.DTREFECUSTO}
        });
    },
    async getRestricoesGrupoPrato ( params ) {
        const SQL = `SELECT V.CDFILIAL AS COD_FILIAL, R.CDSERVFILI AS COD_SERVICO, TO_CHAR( V.DTVIGENCIA, 'MM/YYYY') || CASE WHEN V.DESCRICAO IS NULL THEN '' ELSE ' - ' || V.DESCRICAO END AS VIGENCIA, AG.NMAGRUPGRPRAT AS NOME_AGRUPAMENTO, G.NMGRUPOPRATO AS NOME_DO_GRUPO, G.NMGRUPOPRATO AS NOME_GRUPO, R.NRGRUPOPRATOSERV AS NUM_GRP_PRATO, R.CDPRODUTO AS COD_PRODUTO, P.CDARVPROD AS ARVORE_PRODUTO, P.NMPRODUTO AS NOME_PRODUTO FROM VIGRESTPRODGR V INNER JOIN RESTPRODGR R ON R.NRVIGENCIA = V.NRVIGENCIA AND V.IDTIPORESTRICAO =:P_IDTIPORESTRICAO AND V.NRVIGENCIA =(SELECT NRVIGENCIA FROM (SELECT VV.NRVIGENCIA, VV.DTVIGENCIA, CASE WHEN ( VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM >= :P_DTMESCARD AND VV.DTVIGENCIA <= :P_DTMESCARD ) THEN 0 WHEN ( VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM <= :P_DTMESCARD ) THEN 2 WHEN VV.DTVIGENCIA <= :P_DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGRESTPRODGR VV WHERE :P_CDFILIAL = VV.CDFILIAL AND VV.IDTIPORESTRICAO = :P_IDTIPORESTRICAO ORDER BY ORDEM, DTVIGENCIA DESC ) WHERE ROWNUM = 1) INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = R.NRGRUPOPRATOSERV AND GS.CDSERVFILI =:P_CDSERVFILI INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.IDTIPORESTRICAO =:P_IDTIPORESTRICAO AND G.CDFILIAL =:P_CDFILIAL LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = GS.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO =:P_IDTIPORESTRICAO INNER JOIN PRODUTO P ON P.CDPRODUTO = R.CDPRODUTO ORDER BY AG.NMAGRUPGRPRAT, G.NMGRUPOPRATO, P.CDARVPROD`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMESCARD:        {val: params.DTMESCARD},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO}
        });
    },
    async getRestricoesPratoCombinacao ( params ) {
        const SQL = `SELECT COD_FILIAL, COD_SERVICO, COD_GRP_TIP_PRATO, NOME_GRUPO_TIPO_PRATO, TIPO, CASE WHEN TIPO = 'Grupo' THEN DESC_GRUPO_DE_PRATO ELSE DESC_PRATO END AS DESCRICAO, COD_PRATO, NRGRUPOPRATOSERV, COD_GRP_TIP_PRATO_COMB, NOME_GRUPO_TIPO_PRATO_COMB, TIPO_COMB, CASE WHEN TIPO_COMB = 'Grupo' THEN DESC_GRUPO_DE_PRATO_COMB WHEN TIPO_COMB = 'Prato' THEN DESC_PRATO_COMB ELSE DESC_TIPO_PRATO_COMB END AS DESCRICAO_COMB, COD_PRATO_COMB, NRGRUPOPRATOSERV_COMB, CDTIPOPRAT, SEQ FROM(SELECT C.CDFILIAL AS COD_FILIAL, C.CDSERVFILI AS COD_SERVICO, C.CDFILAUXPRAT || C.CDPRATO AS COD_PRATO, C.CDGRTIPOPR AS COD_GRP_TIP_PRATO, G1.NMGRTIPOPR AS NOME_GRUPO_TIPO_PRATO, CASE WHEN C.CDFILAUXPRAT IS NOT NULL THEN 'Prato' ELSE 'Grupo' END AS TIPO, CASE WHEN P1.CDFILAUXPRAT = 'P' THEN 'Padrão' ELSE 'Local' END ||' - '|| C.CDPRATO ||' - '||P1.NMPRATO AS DESC_PRATO, C.NRGRUPOPRATOSERV, GS1.CDGRUPOPRATO ||' - '||GS1.NMGRUPOPRATO AS DESC_GRUPO_DE_PRATO, I.CDFILAUXPRAT || I.CDPRATO AS COD_PRATO_COMB, I.CDGRTIPOPR AS COD_GRP_TIP_PRATO_COMB, G2.NMGRTIPOPR AS NOME_GRUPO_TIPO_PRATO_COMB, CASE WHEN I.CDFILAUXPRAT IS NOT NULL THEN 'Prato' WHEN I.NRGRUPOPRATOSERV IS NOT NULL THEN 'Grupo' ELSE 'Coluna' END AS TIPO_COMB, CASE WHEN P2.CDFILAUXPRAT = 'P' THEN 'Padrão' ELSE 'Local' END ||' - '|| I.CDPRATO ||' - '||P2.NMPRATO AS DESC_PRATO_COMB, I.NRGRUPOPRATOSERV AS NRGRUPOPRATOSERV_COMB, GS2.CDGRUPOPRATO ||' - '||GS2.NMGRUPOPRATO AS DESC_GRUPO_DE_PRATO_COMB, I.CDTIPOPRAT AS CDTIPOPRAT, I.CDTIPOPRAT ||' - '||TP.NMTIPOPRAT AS DESC_TIPO_PRATO_COMB, RTRIM( XMLAGG(XMLELEMENT( E, CC.NRSEQCNFCARD, ';') ).EXTRACT('//text()').GETCLOBVAL(), ',' ) AS SEQ FROM COMBGRUPOTPPRAT C INNER JOIN ITCOMBGRUPOTPPRAT I ON I.NRCOMBGRUPOTPPRAT = C.NRCOMBGRUPOTPPRAT LEFT JOIN PRATO P1 ON P1.CDFILAUXPRAT = C.CDFILAUXPRAT AND P1.CDPRATO = C.CDPRATO INNER JOIN GRTIPOPR G1 ON G1.CDGRTIPOPR = C.CDGRTIPOPR LEFT JOIN(SELECT GS.NRGRUPOPRATOSERV, G.CDGRUPOPRATO, G.NMGRUPOPRATO FROM GRUPOPRATOSERV GS INNER JOIN GRUPOPRATO G ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO AND GS.CDSERVFILI =:P_CDSERVFILI AND G.CDFILIAL =:P_CDFILIAL AND G.IDTIPORESTRICAO =:P_IDTIPORESTRICAO ) GS1 ON GS1.NRGRUPOPRATOSERV = C.NRGRUPOPRATOSERV INNER JOIN GRTIPOPR G2 ON G2.CDGRTIPOPR = I.CDGRTIPOPR LEFT JOIN PRATO P2 ON P2.CDFILAUXPRAT = I.CDFILAUXPRAT AND P2.CDPRATO = I.CDPRATO LEFT JOIN (SELECT GS.NRGRUPOPRATOSERV, G.CDGRUPOPRATO, G.NMGRUPOPRATO FROM GRUPOPRATOSERV GS INNER JOIN GRUPOPRATO G ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO AND GS.CDSERVFILI =:P_CDSERVFILI AND G.CDFILIAL =:P_CDFILIAL AND G.IDTIPORESTRICAO =:P_IDTIPORESTRICAO ) GS2 ON GS2.NRGRUPOPRATOSERV = I.NRGRUPOPRATOSERV LEFT JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = I.CDTIPOPRAT LEFT JOIN (SELECT DISTINCT CC.NRSEQCNFCARD, TP.CDGRTIPOPR FROM CONFCARDGRUPOPRATO CC INNER JOIN CONFCARD CNF ON CC.CDFILIAL = CNF.CDFILIAL AND CC.CDSERVFILI = CNF.CDSERVFILI AND CC.NRSEQCNFCARD = CNF.NRSEQCNFCARD INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = CNF.CDTIPOPRAT WHERE CNF.CDFILIAL =:P_CDFILIAL AND CNF.CDSERVFILI =:P_CDSERVFILI ) CC ON CC.CDGRTIPOPR = I.CDGRTIPOPR WHERE I.IDCOMBINACAO = 'RESTRICAO' AND C.CDFILIAL =:P_CDFILIAL AND C.CDSERVFILI =:P_CDSERVFILI GROUP BY C.CDFILIAL, C.CDSERVFILI, C.CDFILAUXPRAT, C.CDPRATO, C.CDGRTIPOPR, G1.NMGRTIPOPR, CASE WHEN C.CDFILAUXPRAT IS NOT NULL THEN 'Prato' ELSE 'Grupo' END, CASE WHEN P1.CDFILAUXPRAT = 'P' THEN 'Padrão' ELSE 'Local' END ||' - '|| C.CDPRATO ||' - '||P1.NMPRATO, C.NRGRUPOPRATOSERV, GS1.CDGRUPOPRATO, GS1.NMGRUPOPRATO, I.CDFILAUXPRAT, I.CDPRATO, I.CDGRTIPOPR, G2.NMGRTIPOPR, CASE WHEN I.CDFILAUXPRAT IS NOT NULL THEN 'Prato' WHEN I.NRGRUPOPRATOSERV IS NOT NULL THEN 'Grupo' ELSE 'Coluna' END, CASE WHEN P2.CDFILAUXPRAT = 'P' THEN 'Padrão' ELSE 'Local' END ||' - '|| I.CDPRATO ||' - '||P2.NMPRATO, I.NRGRUPOPRATOSERV, GS2.CDGRUPOPRATO, GS2.NMGRUPOPRATO, I.CDTIPOPRAT, TP.NMTIPOPRAT ) ORDER BY COD_FILIAL, COD_SERVICO, COD_GRP_TIP_PRATO, TIPO, CASE WHEN TIPO = 'Grupo' THEN DESC_GRUPO_DE_PRATO ELSE DESC_PRATO END, COD_GRP_TIP_PRATO_COMB, TIPO_COMB, CASE WHEN TIPO_COMB = 'Grupo' THEN DESC_GRUPO_DE_PRATO_COMB WHEN TIPO_COMB = 'Prato' THEN DESC_PRATO_COMB ELSE DESC_TIPO_PRATO_COMB END`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO}
        });
    },
    async getRestricoesFlexRules ( params ) {
        const SQL = `SELECT NRORDEMREGRA,IDFLEXIVEL,DSREGRAFLEX FROM OTMREGRAFLEX ORDER BY NRORDEMREGRA`;
        return this.execQuery( SQL, {});
    },
    async getTipoPrato ( params ) {
        const SQL = `SELECT TP.CDTIPOPRAT AS COD_TIP_PRATO, TP.NMTIPOPRAT AS NOM_TIP_PRATO, TP.CDGRTIPOPR AS COD_GRP_TIP_PRATO, GT.NMGRTIPOPR AS NOM_GRP_TIP_PRATO FROM TIPOPRAT TP LEFT JOIN GRTIPOPR GT ON TP.CDGRTIPOPR = GT.CDGRTIPOPR`;
        return this.execQuery( SQL, {});
    },
    async getUtilizacaoPratoSemana ( params ) {
        const SQL = `SELECT V.CDFILIAL AS COD_FILIAL, U.CDSERVFILI AS COD_SERVICO, TO_CHAR(V.DTVIGENCIA, 'MM/YYYY') || CASE WHEN V.DESCRICAO IS NULL THEN '' ELSE ' - ' || V.DESCRICAO END AS VIGENCIA, U.NRGRUPOPRATOSERV AS NUM_GRP_PRATO, AG.NMAGRUPGRPRAT AS NOME_AGRUPAMENTO, G.NMGRUPOPRATO AS NOME_GRUPO, G.NMGRUPOPRATO AS NOME_GRUPO_PRATO, DECODE(IDUTILPRATOSERVSEG, 'OBRIGATORIO', 'O', 'RESTRICAO', 'R', 'P') AS IND_SEGUNDA, DECODE(IDUTILPRATOSERVTER, 'OBRIGATORIO', 'O', 'RESTRICAO', 'R', 'P') AS IND_TERCA, DECODE(IDUTILPRATOSERVQUA, 'OBRIGATORIO', 'O', 'RESTRICAO', 'R', 'P') AS IND_QUARTA, DECODE(IDUTILPRATOSERVQUI, 'OBRIGATORIO', 'O', 'RESTRICAO', 'R', 'P') AS IND_QUINTA, DECODE(IDUTILPRATOSERVSEX, 'OBRIGATORIO', 'O', 'RESTRICAO', 'R', 'P') AS IND_SEXTA, DECODE(IDUTILPRATOSERVSAB, 'OBRIGATORIO', 'O', 'RESTRICAO', 'R', 'P') AS IND_SABADO, DECODE(IDUTILPRATOSERVDOM, 'OBRIGATORIO', 'O', 'RESTRICAO', 'R', 'P') AS IND_DOMINGO, DECODE(IDUTILPRATOSERVPRIM, 'OBRIGATORIO', 'O', 'RESTRICAO', 'R', 'P') AS IDPRIMSEM, DECODE(IDUTILPRATOSERVSEM2, 'OBRIGATORIO', 'O', 'RESTRICAO', 'R', 'P') AS IDSEMANA2, DECODE(IDUTILPRATOSERVSEM3, 'OBRIGATORIO', 'O', 'RESTRICAO', 'R', 'P') AS IDSEMANA3, DECODE(IDUTILPRATOSERVSEM4, 'OBRIGATORIO', 'O', 'RESTRICAO', 'R', 'P') AS IDSEMANA4, DECODE(IDUTILPRATOSERVULT, 'OBRIGATORIO', 'O', 'RESTRICAO', 'R', 'P') AS IDULTMSEM, DECODE(IDUTILPRATOSERVFERI, 'OBRIGATORIO', 'O', 'RESTRICAO', 'R', 'P') AS IDFERIADO FROM VIGUTILPRATOSERV V INNER JOIN UTILPRATOSERV U ON V.NRVIGENCIA = U.NRVIGENCIA AND V.IDTIPORESTRICAO = :P_IDTIPORESTRICAO AND V.NRVIGENCIA =(SELECT NRVIGENCIA FROM(SELECT VV.NRVIGENCIA, VV.DTVIGENCIA, VV.DTVIGENCIAFIM, CASE WHEN (VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM >= :P_DTMESCARD AND VV.DTVIGENCIA <= :P_DTMESCARD) THEN 0 WHEN (VV.DTVIGENCIAFIM IS NOT NULL AND VV.DTVIGENCIAFIM <= :P_DTMESCARD) THEN 2 WHEN VV.DTVIGENCIA <= :P_DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGUTILPRATOSERV VV WHERE :P_CDFILIAL = VV.CDFILIAL AND VV.IDTIPORESTRICAO = :P_IDTIPORESTRICAO ORDER BY ORDEM, DTVIGENCIA DESC) WHERE ROWNUM = 1 ) INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = U.NRGRUPOPRATOSERV AND GS.CDSERVFILI = :P_CDSERVFILI INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.CDFILIAL = :P_CDFILIAL AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = GS.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO = :P_IDTIPORESTRICAO ORDER BY AG.NMAGRUPGRPRAT, G.NMGRUPOPRATO`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMESCARD:        {val: params.DTMESCARD},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO}
        });
    },
    async getCalendario ( params ){
        const SQL = `SELECT DTDATA, QTDE_COMENSAIS, CASE WHEN IDCOMCUSPERC = 'C' THEN QTDE_COMENSAIS_CLI ELSE QTDE_COMENSAIS END AS COMENSAIS_DIVISAO, QTDE_COMENSAIS_CLI,QTDE_COMENSAIS_FUN,QTDE_COMENSAIS_DOA, CASE WHEN DTFERIFILI IS NOT NULL THEN 'S' ELSE 'N' END AS IDFERIADO, CASE WHEN IDCARDIAUTIL = 'N' THEN 'S' WHEN PRIOR QTDE_COMENSAIS = 0 THEN 'N' ELSE(SELECT DECODE(COUNT(1), 1,'N','S') FROM FERIFILI WHERE CDFILIAL = :P_CDFILIAL AND DTFERIFILI = DTDATA - 1 ) END AS PER_PREPREPARO FROM (SELECT DTMES, DTDATA, NVL(PF.IDCOMCUSPERC,'T') AS IDCOMCUSPERC, CASE WHEN DTFERIFILI IS NOT NULL THEN NVL(QTCOMESTIFER, 0) WHEN TO_CHAR(DTDATA, 'D') = '1' THEN NVL(QTCOMESTIDOM, 0) WHEN TO_CHAR(DTDATA, 'D') = '2' THEN NVL(QTCOMESTISEG, 0) WHEN TO_CHAR(DTDATA, 'D') = '3' THEN NVL(QTCOMESTITER, 0) WHEN TO_CHAR(DTDATA, 'D') = '4' THEN NVL(QTCOMESTIQUA, 0) WHEN TO_CHAR(DTDATA, 'D') = '5' THEN NVL(QTCOMESTIQUI, 0) WHEN TO_CHAR(DTDATA, 'D') = '6' THEN NVL(QTCOMESTISEX, 0) WHEN TO_CHAR(DTDATA, 'D') = '7' THEN NVL(QTCOMESTISAB, 0) END AS QTDE_COMENSAIS, CASE WHEN DTFERIFILI IS NOT NULL THEN NVL(QTCLIESTIFER, 0) WHEN TO_CHAR(DTDATA, 'D') = '1' THEN NVL(QTCLIESTIDOM, 0) WHEN TO_CHAR(DTDATA, 'D') = '2' THEN NVL(QTCLIESTISEG, 0) WHEN TO_CHAR(DTDATA, 'D') = '3' THEN NVL(QTCLIESTITER, 0) WHEN TO_CHAR(DTDATA, 'D') = '4' THEN NVL(QTCLIESTIQUA, 0) WHEN TO_CHAR(DTDATA, 'D') = '5' THEN NVL(QTCLIESTIQUI, 0) WHEN TO_CHAR(DTDATA, 'D') = '6' THEN NVL(QTCLIESTISEX, 0) WHEN TO_CHAR(DTDATA, 'D') = '7' THEN NVL(QTCLIESTISAB, 0) END AS QTDE_COMENSAIS_CLI, CASE WHEN DTFERIFILI IS NOT NULL THEN NVL(QTFUNESTIFER, 0) WHEN TO_CHAR(DTDATA, 'D') = '1' THEN NVL(QTFUNESTIDOM, 0) WHEN TO_CHAR(DTDATA, 'D') = '2' THEN NVL(QTFUNESTISEG, 0) WHEN TO_CHAR(DTDATA, 'D') = '3' THEN NVL(QTFUNESTITER, 0) WHEN TO_CHAR(DTDATA, 'D') = '4' THEN NVL(QTFUNESTIQUA, 0) WHEN TO_CHAR(DTDATA, 'D') = '5' THEN NVL(QTFUNESTIQUI, 0) WHEN TO_CHAR(DTDATA, 'D') = '6' THEN NVL(QTFUNESTISEX, 0) WHEN TO_CHAR(DTDATA, 'D') = '7' THEN NVL(QTFUNESTISAB, 0) END AS QTDE_COMENSAIS_FUN, CASE WHEN DTFERIFILI IS NOT NULL THEN NVL(QTDOAESTIFER, 0) WHEN TO_CHAR(DTDATA, 'D') = '1' THEN NVL(QTDOAESTIDOM, 0) WHEN TO_CHAR(DTDATA, 'D') = '2' THEN NVL(QTDOAESTISEG, 0) WHEN TO_CHAR(DTDATA, 'D') = '3' THEN NVL(QTDOAESTITER, 0) WHEN TO_CHAR(DTDATA, 'D') = '4' THEN NVL(QTDOAESTIQUA, 0) WHEN TO_CHAR(DTDATA, 'D') = '5' THEN NVL(QTDOAESTIQUI, 0) WHEN TO_CHAR(DTDATA, 'D') = '6' THEN NVL(QTDOAESTISEX, 0) WHEN TO_CHAR(DTDATA, 'D') = '7' THEN NVL(QTDOAESTISAB, 0) END AS QTDE_COMENSAIS_DOA, P.IDCARDIAUTIL, F.DTFERIFILI FROM DATA CROSS JOIN SERVICO S LEFT JOIN PARAMPLANEJA P ON P.CDFILIAL = :P_CDFILIAL LEFT JOIN PARAMFILIAL PF ON PF.CDFILIAL = :P_CDFILIAL LEFT JOIN FERIFILI F ON F.CDFILIAL = :P_CDFILIAL AND F.DTFERIFILI = DTDATA WHERE (DTDATA BETWEEN TO_DATE(:P_DTMES)-1 AND ADD_MONTHS(TO_DATE(:P_DTMES),1)-1) AND S.CDFILIAL = :P_CDFILIAL AND S.CDSERVFILI = :P_CDSERVFILI ) WHERE ((IDCARDIAUTIL = 'N') OR (IDCARDIAUTIL = 'S' AND DTFERIFILI IS NULL)) AND QTDE_COMENSAIS > 0 AND DTMES = :P_DTMES START WITH DTDATA = TO_DATE(:P_DTMES)-1 CONNECT BY PRIOR DTDATA = DTDATA -1 ORDER BY 1`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMES:            {val: params.DTMES}
        });
    },
    async getComplexidade (params) {
        return new Promise(
            (resolve, reject) => {
                resolve({
                    status: "SUCCESS",
                    data: []
                })
            }
        );
    },
    async getConsumoPrato ( params ) {
        const SQL = `SELECT CDFILAUXPRAT || CDPRATO AS COD_PRATO, VRPECONSCARD FROM(SELECT CP.CDFILAUXPRAT, CP.CDPRATO, ROUND(AVG(CP.VRPECONSCARD),2) AS VRPECONSCARD FROM COMPCARD CP INNER JOIN CARDADIA C ON C.CDFILIAL = CP.CDFILIAL AND C.NRCARDAPIO = CP.NRCARDAPIO INNER JOIN SERVICO S ON S.CDFILIAL = C.CDFILIAL AND S.CDSERVFILI = C.CDSERVFILI WHERE C.CDFILIAL = :P_CDFILIAL AND C.CDSERVFILI = :P_CDSERVFILI AND (C.DTMESCARD BETWEEN ADD_MONTHS(TO_DATE(:P_DTMESCARD),(NVL(NRMESHISTCONS,6)*-1)) AND TO_DATE(:P_DTMESCARD)) AND NOT EXISTS (SELECT H.CDPRATO FROM HISTCONSUMOPRATO H WHERE H.CDFILIAL = :P_CDFILIAL AND H.CDSERVFILI = :P_CDSERVFILI AND H.CDFILAUXPRAT = CP.CDFILAUXPRAT AND H.CDPRATO = CP.CDPRATO ) GROUP BY CP.CDFILAUXPRAT, CP.CDPRATO UNION ALL SELECT H.CDFILAUXPRAT, H.CDPRATO, H.VRPECONSCARD FROM HISTCONSUMOPRATO H WHERE H.CDFILIAL = :P_CDFILIAL AND H.CDSERVFILI = :P_CDSERVFILI ) H ORDER BY H.CDPRATO`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMESCARD:        {val: params.DTMESCARD}
        });
    },
    async getSeqOrder (params) {
        const SQL = `SELECT CF.NRSEQCNFCARD, TP.NMTIPOPRAT || CASE WHEN CF.DSCOMPLTP IS NULL THEN '' ELSE '(' || CF.DSCOMPLTP || ')' END AS NMTIPOPRAT, CF.IDOPCCNFCARD FROM CONFCARD CF INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = CF.CDTIPOPRAT LEFT JOIN CONFCARDMESATIVO CFA ON CFA.CDFILIAL = CF.CDFILIAL AND CFA.CDSERVFILI = CF.CDSERVFILI AND CFA.NRSEQCNFCARD = CF.NRSEQCNFCARD WHERE CF.CDFILIAL = :P_CDFILIAL AND CF.CDSERVFILI = :P_CDSERVFILI AND SUBSTR(NVL(CFA.IDMESATIVO,'SSSSSSSSSSSS'), TO_NUMBER(TO_CHAR(TO_DATE(:P_DTMESCARD), 'MM')),1) = 'S' ORDER BY CF.NRORDCNFCARD`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMESCARD:        {val: params.DTMESCARD}
        });
    },
    async getTpCorte ( params ) {
        const SQL = `SELECT CDAPRERECE, NMAPRERECE FROM APRERECE UNION ALL SELECT NULL AS CDAPRERECE, 'SEM TIPO DE CORTE' AS NMAPRERECE FROM DUAL`;
        return this.execQuery( SQL, {});
    },
    async getMdPreparo ( params ) {
        const SQL = `SELECT NRFORMAPREPARO, NMFORMAPREPARO FROM FORMAPREPARO UNION ALL SELECT NULL AS NRFORMAPREPARO, 'SEM MODO DE PREPARO' AS NMFORMAPREPARO FROM DUAL`;
        return this.execQuery( SQL, {});
    },
    async getIngPrinc ( params ) {
        const SQL = `SELECT NRINGREDIENTEPRINC, NMINGREDIENTEPRINC FROM INGREDIENTEPRINC UNION ALL SELECT NULL AS NRINGREDIENTEPRINC, 'SEM INGREDIENTE PRINCIPAL' AS NMINGREDIENTEPRINC FROM DUAL`;
        return this.execQuery( SQL, {});
    },
    async getCongDays (params) {
        const SQL = `SELECT DT.COD_FILIAL, DT.COD_SERVICO, DT.DTDATA, C.QTDE_COMENSAIS, C.COMENSAIS_DIVISAO, C.QTDE_COMENSAIS_CLI, C.QTDE_COMENSAIS_FUN, C.QTDE_COMENSAIS_DOA, DT.NUM_SEQUENCIA, DT.CDTIPOPRAT, C.NUM_GRP_PRATO, C.NOME_GRP_PRATO, C.UTILIZACAO, C.CDFILAUXPRAT, C.COD_PRATO, C.NOM_PRATO, C.VRPECONSCARD, C.VLR_CUSTO_PRATO, C.COR_PRATO, C.COMPLEX_PRATO, C.PRE_PREPARO, C.INGR_PRINCIPAL_1, C.NM_INGR_PRINCIPAL_1, C.INGR_PRINCIPAL_2, C.NM_INGR_PRINCIPAL_2, C.TP_CORTE, C.NM_TP_CORTE, C.MODO_PREPARO, C.NM_MODO_PREPARO, C.ATRATIVIDADE, C.NMATRATIVIDADE, C.VRPESOATRATIVIDADE, C.NOM_PRODUTO, C.COD_ARV_PRODUTO, C.COD_PRODUTO, C.GRAMAGEM FROM(SELECT CNF.CDFILIAL AS COD_FILIAL, CNF.CDSERVFILI AS COD_SERVICO, DTDATA, CNF.NRORDCNFCARD, CNF.NRSEQCNFCARD AS NUM_SEQUENCIA, CNF.CDTIPOPRAT FROM (SELECT DTDATA FROM DATA DT WHERE (DT.DTDATA BETWEEN TO_DATE(:P_DTMESCARD) AND TO_DATE(:P_DTMESCARD) + (:P_DIAS_CONG - 1)) UNION SELECT DTDATA FROM DATA DT INNER JOIN CARDIAEVENTO E ON E.DTEVENTO = DT.DTDATA WHERE E.CDFILIAL = :P_CDFILIAL AND E.CDSERVFILI = :P_CDSERVFILI AND TRUNC(E.DTEVENTO, 'MM') = :P_DTMESCARD) DT CROSS JOIN CONFCARD CNF LEFT JOIN CONFCARDMESATIVO CFA ON CNF.CDFILIAL = CFA.CDFILIAL AND CNF.CDSERVFILI = CFA.CDSERVFILI AND CNF.NRSEQCNFCARD = CFA.NRSEQCNFCARD WHERE CNF.CDFILIAL = :P_CDFILIAL AND CNF.CDSERVFILI = :P_CDSERVFILI AND SUBSTR(NVL(CFA.IDMESATIVO,'SSSSSSSSSSSS'), TO_NUMBER(TO_CHAR(TO_DATE(:P_DTMESCARD), 'MM')),1) = 'S' ) DT LEFT JOIN (SELECT C.CDFILIAL AS COD_FILIAL, C.CDSERVFILI AS COD_SERVICO, C.DTCARDAPIO AS DTDATA, C.QTCOMESTICAR AS QTDE_COMENSAIS, CASE WHEN NVL(PF.IDCOMCUSPERC,'T') = 'C' THEN C.QTCLIESTICAR ELSE C.QTCOMESTICAR END AS COMENSAIS_DIVISAO, C.QTCLIESTICAR AS QTDE_COMENSAIS_CLI, C.QTFUNESTICAR AS QTDE_COMENSAIS_FUN, C.QTDOAESTICAR AS QTDE_COMENSAIS_DOA, CP.NRORDPRATCAR, CP.NRSEQPRATCAR AS NUM_SEQUENCIA, CP.CDTIPOPRAT, GS.NRGRUPOPRATOSERV AS NUM_GRP_PRATO, G.NMGRUPOPRATO AS NOME_GRP_PRATO, 'O' AS UTILIZACAO, CP.CDFILAUXPRAT, CP.CDFILAUXPRAT || CP.CDPRATO AS COD_PRATO, P.NMPRATO AS NOM_PRATO, CP.VRPECONSCARD, CUSTO_PRATO(:P_CDFILIAL, :P_CDFILIAL, :P_NRTIPOCUSTO, :P_DTREFECUSTO, CP.CDFILAUXPRAT, CP.CDPRATO, :P_CDSERVFILI) VLR_CUSTO_PRATO, PTF.IDCOR AS COR_PRATO, NVL(PTF.IDCOMPLEXIDADE, 'BAIXA') AS COMPLEX_PRATO, NVL(PTF.IDPREPREPARO, 'N') AS PRE_PREPARO, PTF.NRINGREDIENTEPRINC1 AS INGR_PRINCIPAL_1, ING1.NMINGREDIENTEPRINC AS NM_INGR_PRINCIPAL_1, PTF.NRINGREDIENTEPRINC2 AS INGR_PRINCIPAL_2, ING2.NMINGREDIENTEPRINC AS NM_INGR_PRINCIPAL_2, PTF.CDAPRERECE AS TP_CORTE, AP.NMAPRERECE AS NM_TP_CORTE, PTF.NRFORMAPREPARO AS MODO_PREPARO, FP.NMFORMAPREPARO AS NM_MODO_PREPARO, PTF.NRATRATIVIDADEPRATO AS ATRATIVIDADE, AT.NMATRATIVIDADEPRATO AS NMATRATIVIDADE, AT.VRPESOATRATIVIDADE, PR.NMPRODUTO AS NOM_PRODUTO, PR.CDARVPROD AS COD_ARV_PRODUTO, REC.CDPRODUTO AS COD_PRODUTO, REC.QTPRODUTO AS GRAMAGEM FROM (SELECT CDFILIAL, CDSERVFILI, NRCARDAPIO, DTCARDAPIO, DTMESCARD, QTCOMESTICAR, QTCLIESTICAR, QTFUNESTICAR, QTDOAESTICAR FROM CARDADIA C WHERE C.CDFILIAL = :P_CDFILIAL AND C.CDSERVFILI =:P_CDSERVFILI AND C.DTMESCARD = :P_DTMESCARD AND (C.DTCARDAPIO BETWEEN C.DTMESCARD AND C.DTMESCARD + (:P_DIAS_CONG - 1 )) AND C.QTCOMESTICAR > 0 AND NOT EXISTS(SELECT NRCARDIAEVENTO FROM CARDIAEVENTO E WHERE E.CDFILIAL = C.CDFILIAL AND E.CDSERVFILI = C.CDSERVFILI AND E.DTEVENTO = C.DTCARDAPIO ) UNION SELECT C.CDFILIAL, C.CDSERVFILI, C.NRCARDAPIO, C.DTCARDAPIO, C.DTMESCARD, NVL(E.QTCLIESTICAR,0) + NVL(E.QTFUNESTICAR,0) + NVL(E.QTDOAESTICAR,0) AS QTCOMESTICAR, E.QTCLIESTICAR, E.QTFUNESTICAR, E.QTDOAESTICAR FROM CARDADIA C INNER JOIN CARDIAEVENTO E ON E.CDFILIAL = C.CDFILIAL AND E.CDSERVFILI = C.CDSERVFILI AND E.DTEVENTO = C.DTCARDAPIO WHERE C.CDFILIAL = :P_CDFILIAL AND C.CDSERVFILI =:P_CDSERVFILI AND C.DTMESCARD = :P_DTMESCARD AND C.QTCOMESTICAR > 0 ) C INNER JOIN COMPCARD CP ON CP.CDFILIAL = C.CDFILIAL AND CP.NRCARDAPIO = C.NRCARDAPIO INNER JOIN PRATO P ON P.CDFILAUXPRAT = CP.CDFILAUXPRAT AND P.CDPRATO = CP.CDPRATO INNER JOIN GRUPOPRATO G ON G.CDFILIAL =:P_CDFILIAL AND G.IDTIPORESTRICAO =:P_IDTIPORESTRICAO INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO AND GS.CDSERVFILI =:P_CDSERVFILI INNER JOIN ITGRUPOPRATOSERV IT ON IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV AND IT.CDFILAUXPRAT = CP.CDFILAUXPRAT AND IT.CDPRATO = CP.CDPRATO LEFT JOIN PARAMPRATFILIAL PTF ON CP.CDPRATO = PTF.CDPRATO AND CP.CDFILAUXPRAT = PTF.CDFILAUXPRAT AND PTF.CDFILIAL =:P_CDFILIAL LEFT JOIN INGREDIENTEPRINC ING1 ON PTF.NRINGREDIENTEPRINC1 = ING1.NRINGREDIENTEPRINC LEFT JOIN INGREDIENTEPRINC ING2 ON PTF.NRINGREDIENTEPRINC2 = ING2.NRINGREDIENTEPRINC LEFT JOIN ATRATIVIDADEPRATO AT ON AT.NRATRATIVIDADEPRATO = PTF.NRATRATIVIDADEPRATO LEFT JOIN FORMAPREPARO FP ON FP.NRFORMAPREPARO = PTF.NRFORMAPREPARO LEFT JOIN APRERECE AP ON AP.CDAPRERECE = PTF.CDAPRERECE INNER JOIN (SELECT RS.CDPRATO, RS.CDFILAUXPRAT, RS.CDPRODUTO, RS.QTBRUTASE AS QTPRODUTO FROM RECESERV RS LEFT JOIN PRATSERV PS ON PS.CDFILIAL = RS.CDFILIAL AND PS.CDSERVFILI = RS.CDSERVFILI AND PS.CDPRATO = RS.CDPRATO AND PS.CDFILAUXPRAT = RS.CDFILAUXPRAT WHERE RS.CDFILIAL = :P_CDFILIAL AND RS.CDSERVFILI = :P_CDSERVFILI AND NVL(PS.IDATIVO, 'S') = 'S' UNION ALL SELECT RF.CDPRATO, RF.CDFILAUXPRAT, RF.CDPRODUTO, RF.QTBRUTALO AS QTPRODUTO FROM RECEFILI RF LEFT JOIN PRATFILI PF ON PF.CDFILIAL = RF.CDFILIAL AND PF.CDPRATO = RF.CDPRATO AND PF.CDFILAUXPRAT = RF.CDFILAUXPRAT WHERE RF.CDFILIAL = :P_CDFILIAL AND NOT EXISTS(SELECT RS.CDPRATO FROM RECESERV RS WHERE RS.CDFILIAL = :P_CDFILIAL AND RS.CDSERVFILI = :P_CDSERVFILI AND RS.CDPRATO = RF.CDPRATO AND RS.CDFILAUXPRAT = RF.CDFILAUXPRAT) AND NVL(PF.IDATIVO, 'S') = 'S' UNION ALL SELECT RP.CDPRATO, RP.CDFILAUXPRAT, RP.CDPRODUTO, RP.QTBRUTAPR AS QTPRODUTO FROM RECEPADR RP LEFT JOIN PRATPADR PP ON PP.CDPRATO = RP.CDPRATO AND PP.CDFILAUXPRAT = RP.CDFILAUXPRAT INNER JOIN PARAMGERAL PG ON PG.IDUTRPADCA = 'S' WHERE NOT EXISTS(SELECT RF.CDPRATO FROM RECEFILI RF WHERE RF.CDFILIAL = :P_CDFILIAL AND RF.CDPRATO = RP.CDPRATO AND RF.CDFILAUXPRAT = RP.CDFILAUXPRAT ) AND NOT EXISTS(SELECT RS.CDPRATO FROM RECESERV RS WHERE RS.CDFILIAL = :P_CDFILIAL AND RS.CDSERVFILI = :P_CDSERVFILI AND RS.CDPRATO = RP.CDPRATO AND RS.CDFILAUXPRAT = RP.CDFILAUXPRAT ) AND NVL(PP.IDATIVO, 'S') = 'S' ) REC ON REC.CDPRATO = CP.CDPRATO AND REC.CDFILAUXPRAT = CP.CDFILAUXPRAT INNER JOIN PRODUTO PR ON PR.CDPRODUTO = REC.CDPRODUTO LEFT JOIN PARAMFILIAL PF ON PF.CDFILIAL = :P_CDFILIAL ) C ON DT.COD_FILIAL = C.COD_FILIAL AND DT.COD_SERVICO = C.COD_SERVICO AND DT.DTDATA = C.DTDATA AND DT.NRORDCNFCARD = C.NRORDPRATCAR AND DT.CDTIPOPRAT = C.CDTIPOPRAT ORDER BY COD_FILIAL, COD_SERVICO, DTDATA, NUM_SEQUENCIA, COD_PRATO, COD_ARV_PRODUTO`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMESCARD:        {val: params.DTMESCARD},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO},
            P_DTREFECUSTO:      {val: params.DTREFECUSTO},
            P_NRTIPOCUSTO:      {val: params.NRTIPOCUSTO},
            P_DIAS_CONG:        {val: params.DIAS_CONG}
        });
    },
    async getLastWeekProds (params) {
        const SQL = `SELECT C.CDFILIAL AS COD_FILIAL, C.CDSERVFILI AS COD_SERVICO, C.DTCARDAPIO AS DTDATA, GS.NRGRUPOPRATOSERV AS NUM_GRP_PRATO, G.NMGRUPOPRATO AS NOME_GRP_PRATO, CP.CDFILAUXPRAT, CP.CDFILAUXPRAT || CP.CDPRATO AS COD_PRATO, P.NMPRATO AS NOM_PRATO, PTF.IDCOR AS COR_PRATO, NVL(PTF.IDCOMPLEXIDADE,'BAIXA') AS COMPLEX_PRATO, NVL(PTF.IDPREPREPARO,'N') AS PRE_PREPARO, PTF.NRINGREDIENTEPRINC1 AS INGR_PRINCIPAL_1, PTF.NRINGREDIENTEPRINC2 AS INGR_PRINCIPAL_2, PTF.CDAPRERECE AS TP_CORTE, PTF.NRFORMAPREPARO AS MODO_PREPARO, PR.NMPRODUTO AS NOM_PRODUTO, PR.CDARVPROD AS COD_ARV_PRODUTO, REC.CDPRODUTO AS COD_PRODUTO, REC.QTPRODUTO AS GRAMAGEM FROM CARDADIA C INNER JOIN COMPCARD CP ON CP.CDFILIAL = C.CDFILIAL AND CP.NRCARDAPIO = C.NRCARDAPIO INNER JOIN PRATO P ON P.CDFILAUXPRAT = CP.CDFILAUXPRAT AND P.CDPRATO = CP.CDPRATO INNER JOIN GRUPOPRATO G ON G.CDFILIAL =:P_CDFILIAL AND G.IDTIPORESTRICAO =:P_IDTIPORESTRICAO INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO AND GS.CDSERVFILI =:P_CDSERVFILI INNER JOIN ITGRUPOPRATOSERV IT ON IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV AND IT.CDFILAUXPRAT = CP.CDFILAUXPRAT AND IT.CDPRATO = CP.CDPRATO LEFT JOIN PARAMPRATFILIAL PTF ON CP.CDPRATO = PTF.CDPRATO AND CP.CDFILAUXPRAT = PTF.CDFILAUXPRAT AND PTF.CDFILIAL =:P_CDFILIAL INNER JOIN TABLE( RECEITA_UTIL( :P_CDFILIAL, :P_CDSERVFILI, CP.CDPRATO, CP.CDFILAUXPRAT, '0', 'Z') ) REC ON REC.CDPRATO = CP.CDPRATO AND REC.CDFILAUXPRAT = CP.CDFILAUXPRAT INNER JOIN PRODUTO PR ON PR.CDPRODUTO = REC.CDPRODUTO WHERE (C.DTCARDAPIO BETWEEN TO_DATE(:P_DTMESCARD) - :P_DIA_SEMANA AND TO_DATE(:P_DTMESCARD)-1) AND C.CDFILIAL = :P_CDFILIAL AND C.CDSERVFILI = :P_CDSERVFILI AND C.DTMESCARD = ADD_MONTHS(TO_DATE(:P_DTMESCARD),-1) ORDER BY C.CDFILIAL, C.CDSERVFILI, C.DTCARDAPIO, CP.CDFILAUXPRAT, CP.CDPRATO`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMESCARD:        {val: params.DTMESCARD},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO},
            P_DIA_SEMANA:       {val: moment(params.DTMESCARD, 'DD/MM/YYYY').weekday()}
        });
    },
    async getConjuntoPorcao (params){
        const SQL = `SELECT CDGRTIPOPR, VRCONJUNTOPORCAO FROM CONJUNTOPORCAO WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI}
        });
    },
    async getPrato_sem_historico ( params ) {
        const SQL = `SELECT DISTINCT IT.CDFILAUXPRAT||IT.CDPRATO AS COD_PRATO, P.NMPRATO AS NOM_PRATO, CF.IDUTILPERCENTCONS AS TIPO_ERRO FROM CONFCARDGRUPOPRATO CG INNER JOIN CONFCARD CF ON CF.CDFILIAL = CG.CDFILIAL AND CF.CDSERVFILI = CG.CDSERVFILI AND CF.NRSEQCNFCARD = CG.NRSEQCNFCARD LEFT JOIN CONFCARDMESATIVO CFA ON CFA.CDFILIAL = CG.CDFILIAL AND CFA.CDSERVFILI = CG.CDSERVFILI AND CFA.NRSEQCNFCARD = CG.NRSEQCNFCARD INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = CG.NRGRUPOPRATOSERV AND GS.CDSERVFILI = CG.CDSERVFILI INNER JOIN ITGRUPOPRATOSERV IT ON IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.CDFILIAL = CG.CDFILIAL AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO LEFT JOIN PARAMPRATFILIAL PF ON PF.CDFILIAL = CG.CDFILIAL AND PF.CDFILAUXPRAT = IT.CDFILAUXPRAT AND PF.CDPRATO = IT.CDPRATO INNER JOIN PRATO P ON P.CDFILAUXPRAT = IT.CDFILAUXPRAT AND P.CDPRATO = IT.CDPRATO LEFT JOIN(SELECT DISTINCT CP.CDFILAUXPRAT, CP.CDPRATO FROM COMPCARD CP INNER JOIN CARDADIA C ON CP.CDFILIAL = C.CDFILIAL AND CP.NRCARDAPIO = C.NRCARDAPIO AND C.CDFILIAL = :P_CDFILIAL AND C.CDSERVFILI = :P_CDSERVFILI INNER JOIN SERVICO S ON S.CDFILIAL = C.CDFILIAL AND S.CDSERVFILI = C.CDSERVFILI WHERE(C.DTMESCARD BETWEEN ADD_MONTHS(TRUNC(SYSDATE,'MM'),(NVL(NRMESHISTCONS,6)*-1)) AND :P_DTMESCARD) UNION ALL SELECT H.CDFILAUXPRAT, H.CDPRATO FROM HISTCONSUMOPRATO H WHERE H.CDFILIAL = :P_CDFILIAL AND H.CDSERVFILI = :P_CDSERVFILI) H ON H.CDFILAUXPRAT = IT.CDFILAUXPRAT AND H.CDPRATO = IT.CDPRATO WHERE CG.CDFILIAL = :P_CDFILIAL AND CG.CDSERVFILI = :P_CDSERVFILI AND((CF.IDUTILPERCENTCONS = 'HISTORICO' AND H.CDFILAUXPRAT IS NULL) OR(CF.IDUTILPERCENTCONS = 'ATRATIVIDADE' AND PF.NRATRATIVIDADEPRATO IS NULL)) AND PRATO_RECEITA_ATIVA(IT.CDPRATO, IT.CDFILAUXPRAT, :P_CDFILIAL, :P_CDSERVFILI) = 'S' AND SUBSTR(NVL(CFA.IDMESATIVO,'SSSSSSSSSSSS'), TO_NUMBER(TO_CHAR(TO_DATE(:P_DTMESCARD), 'MM')),1) = 'S' ORDER BY TIPO_ERRO, NOM_PRATO`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMESCARD:        {val: params.DTMESCARD},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO}
        });
    },
    async getPrato_restricao_de_repeticao_ingrediente ( params ) {
        const SQL = `SELECT R.NRREGRARESTREPET, rtrim(xmlagg(xmlelement(e, T.CDTIPOPRAT, ';')).extract('//text()').getclobval(), ',') as tipoprat, I.NRINGREDIENTEPRINC, NVL(ING.NMINGREDIENTEPRINC,'TODOS') AS NMINGREDIENTEPRINC, I.IDINGREDIENTE1 AS INGR_PRINCIPAL_1, I.IDINGREDIENTE2 AS INGR_PRINCIPAL_2, I.IDMESMODIA, I.IDDIASEGUIDO FROM REGRARESTREPET R INNER JOIN RESTRICAOTPPRAT T ON T.NRREGRARESTREPET = R.NRREGRARESTREPET INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = T.CDTIPOPRAT INNER JOIN RESTRICAOINGRED I ON I.NRREGRARESTREPET = R.NRREGRARESTREPET LEFT JOIN INGREDIENTEPRINC ING ON ING.NRINGREDIENTEPRINC = I.NRINGREDIENTEPRINC WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI GROUP BY R.NRREGRARESTREPET, I.NRINGREDIENTEPRINC, ING.NMINGREDIENTEPRINC, I.IDINGREDIENTE1, I.IDINGREDIENTE2, I.IDMESMODIA, I.IDDIASEGUIDO ORDER BY R.NRREGRARESTREPET`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI}
        });
    },
    async getIncidencia_tp_gp_prato ( params ) {
        const SQL = `SELECT DISTINCT ROUND(CUSTO_TOT_MES/QTCOMENS_TOTAL,5) AS PU_MES, DTMESCARD, CDTIPOPRAT, NMTIPOPRAT, ROUND(CUSTO_TOT_TIPOPRATO/QTCOMENS_TOTAL, 5) AS PU_TPPRAT, ROUND(AVG(CONSUMO_TPPRAT) OVER(PARTITION BY DTMESCARD, CDTIPOPRAT),2) AS CONSUMO_TPPRAT, QTDE_TPPRAT, CDGRUPOPRATO, NMGRUPOPRATO, NRGRUPOPRATOSERV, ROUND(CUSTO_TOT_GRUPOPRATO/QTCOMENS_TOTAL, 5) AS PU_GPPRATO, CONSUMO_GPPRATO, QTDE_GPPRATO, CDPRATO, CDFILAUXPRAT, NMPRATO, ROUND(CUSTO_TOT_PRATO/QTCOMENS_TOTAL,5) AS PU_PRATO, CONSUMO_PRATO, QTDE_PRATO, QT_DTCARDAPIO FROM(SELECT /*CAMPOS DO MES*/ DISTINCT C.DTMESCARD, SUM(((CUSTO_PRATO(C.CDFILIAL, C.CDFILIAL, :P_NRTIPOCUSTO, NVL(CA.DTREFECUSTMES, C.DTCARDAPIO), CP.CDFILAUXPRAT, CP.CDPRATO, C.CDSERVFILI) * ((CP.VRPECONSCARD / 100) * C.QTCOMESTICAR)))) OVER (PARTITION BY C.DTMESCARD) AS CUSTO_TOT_MES, (SELECT SUM(CASE WHEN NVL(PF.IDCOMCUSPERC,'T') = 'C' THEN NVL(CA.QTCLIESTICAR,CA.QTCOMESTICAR) ELSE CA.QTCOMESTICAR END) FROM CARDADIA CA WHERE CA.CDFILIAL = :P_CDFILIAL AND CA.CDSERVFILI = :P_CDSERVFILI AND CA.DTMESCARD = C.DTMESCARD) AS QTCOMENS_TOTAL, COUNT(DISTINCT DTCARDAPIO) OVER(PARTITION BY C.DTMESCARD) AS QT_DTCARDAPIO, /*CAMPOS DO TIPO DE PRATO*/ CP.CDTIPOPRAT, TP.NMTIPOPRAT, SUM(((CUSTO_PRATO(C.CDFILIAL, C.CDFILIAL, :P_NRTIPOCUSTO, NVL(CA.DTREFECUSTMES, C.DTCARDAPIO), CP.CDFILAUXPRAT, CP.CDPRATO, C.CDSERVFILI) * ((CP.VRPECONSCARD / 100) * C.QTCOMESTICAR)))) OVER (PARTITION BY C.DTMESCARD, CP.CDTIPOPRAT) AS CUSTO_TOT_TIPOPRATO, SUM(CP.VRPECONSCARD) OVER (PARTITION BY C.DTCARDAPIO, CP.CDTIPOPRAT) AS CONSUMO_TPPRAT, COUNT(CP.CDPRATO||CP.CDFILAUXPRAT) OVER (PARTITION BY C.DTMESCARD, CP.CDTIPOPRAT) AS QTDE_TPPRAT, /*CAMPOS DO GRUPO DE PRATO*/ NVL(G.CDGRUPOPRATO, '0000000000') AS CDGRUPOPRATO, NVL(G.NMGRUPOPRATO, '----SEM GRUPO----') AS NMGRUPOPRATO, G.NRGRUPOPRATOSERV, SUM(((CUSTO_PRATO(C.CDFILIAL, C.CDFILIAL, :P_NRTIPOCUSTO, NVL(CA.DTREFECUSTMES, C.DTCARDAPIO), CP.CDFILAUXPRAT, CP.CDPRATO, C.CDSERVFILI) * ((CP.VRPECONSCARD / 100) * C.QTCOMESTICAR)))) OVER (PARTITION BY C.DTMESCARD, CP.CDTIPOPRAT, G.NRGRUPOPRATOSERV) AS CUSTO_TOT_GRUPOPRATO, COUNT(CP.CDPRATO||CP.CDFILAUXPRAT) OVER (PARTITION BY C.DTMESCARD, CP.CDTIPOPRAT, G.NRGRUPOPRATOSERV) AS QTDE_GPPRATO, ROUND(AVG(CP.VRPECONSCARD) OVER (PARTITION BY C.DTMESCARD, CP.CDTIPOPRAT, G.NRGRUPOPRATOSERV), 2) AS CONSUMO_GPPRATO, /*CAMPOS DE PRATO*/ CP.CDPRATO, CP.CDFILAUXPRAT, P.NMPRATO, SUM(((CUSTO_PRATO(C.CDFILIAL, C.CDFILIAL, :P_NRTIPOCUSTO, NVL(CA.DTREFECUSTMES, C.DTCARDAPIO), CP.CDFILAUXPRAT, CP.CDPRATO, C.CDSERVFILI) * ((CP.VRPECONSCARD / 100) * C.QTCOMESTICAR)))) OVER (PARTITION BY C.DTMESCARD, CP.CDTIPOPRAT, CP.CDPRATO, CP.CDFILAUXPRAT) AS CUSTO_TOT_PRATO, ROUND(AVG(CP.VRPECONSCARD) OVER (PARTITION BY C.DTMESCARD, CP.CDTIPOPRAT, TP.NMTIPOPRAT, CP.CDPRATO||CP.CDFILAUXPRAT),2) AS CONSUMO_PRATO, COUNT(CP.CDPRATO||CP.CDFILAUXPRAT) OVER (PARTITION BY C.DTMESCARD, CP.CDTIPOPRAT, TP.NMTIPOPRAT, CP.CDPRATO||CP.CDFILAUXPRAT) AS QTDE_PRATO FROM CARDADIA C INNER JOIN COMPCARD CP ON C.CDFILIAL = CP.CDFILIAL AND C.NRCARDAPIO = CP.NRCARDAPIO INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = CP.CDTIPOPRAT LEFT JOIN CARDAMES CA ON CA.CDFILIAL = C.CDFILIAL AND CA.CDSERVFILI = C.CDSERVFILI AND CA.DTMESCARDAPIO = C.DTMESCARD INNER JOIN PRATO P ON P.CDFILAUXPRAT = CP.CDFILAUXPRAT AND P.CDPRATO = CP.CDPRATO LEFT JOIN (SELECT G.CDGRUPOPRATO, GS.NRGRUPOPRATOSERV, G.NMGRUPOPRATO, IT.CDFILAUXPRAT, IT.CDPRATO FROM GRUPOPRATO G INNER JOIN GRUPOPRATOSERV GS ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.CDFILIAL = :P_CDFILIAL AND GS.CDSERVFILI = :P_CDSERVFILI AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO INNER JOIN ITGRUPOPRATOSERV IT ON IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV ) G ON G.CDFILAUXPRAT = CP.CDFILAUXPRAT AND G.CDPRATO = CP.CDPRATO LEFT JOIN PARAMFILIAL PF ON PF.CDFILIAL = :P_CDFILIAL WHERE C.CDFILIAL = :P_CDFILIAL AND C.CDSERVFILI = :P_CDSERVFILI AND (C.DTMESCARD BETWEEN ADD_MONTHS(TO_DATE(:P_DTMESCARD), -3) AND :P_DTMESCARD) ) ORDER BY NMTIPOPRAT, CDTIPOPRAT, DTMESCARD, NMGRUPOPRATO, NMPRATO`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_DTMESCARD:        {val: params.DTMESCARD},
            P_NRTIPOCUSTO:      {val: params.NRTIPOCUSTO},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO}
        });
    },
    async get_pratos_sem_grupo ( params ) {
        const SQL = `SELECT CDFILAUXPRAT||CDPRATO AS COD_PRATO, NMPRATO AS NOM_PRATO FROM(SELECT P.CDFILAUXPRAT, P.CDPRATO, P.NMPRATO, G.NRGRUPOPRATOSERV,(SELECT DISTINCT 'S' AS IDTIPO FROM RECESERV LEFT JOIN PRATSERV ON PRATSERV.CDPRATO = RECESERV.CDPRATO AND PRATSERV.CDFILAUXPRAT = RECESERV.CDFILAUXPRAT AND PRATSERV.CDFILIAL = RECESERV.CDFILIAL AND PRATSERV.CDSERVFILI = RECESERV.CDSERVFILI WHERE ( RECESERV.CDFILAUXPRAT = P.CDFILAUXPRAT) AND ( RECESERV.CDPRATO = P.CDPRATO) AND ( RECESERV.CDFILIAL =:P_CDFILIAL ) AND ( RECESERV.CDSERVFILI =:P_CDSERVFILI ) AND (NVL(PRATSERV.IDATIVO,'S') = 'S') UNION ALL SELECT DISTINCT 'L' AS IDTIPO FROM RECEFILI LEFT JOIN PRATFILI ON PRATFILI.CDPRATO = RECEFILI.CDPRATO AND PRATFILI.CDFILAUXPRAT = RECEFILI.CDFILAUXPRAT AND PRATFILI.CDFILIAL = RECEFILI.CDFILIAL WHERE ( RECEFILI.CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( RECEFILI.CDPRATO = P.CDPRATO ) AND ( RECEFILI.CDFILIAL =:P_CDFILIAL ) AND (NVL(PRATFILI.IDATIVO,'S') = 'S') AND NOT EXISTS (SELECT CDPRATO FROM RECESERV WHERE ( CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( CDPRATO = P.CDPRATO ) AND ( CDFILIAL =:P_CDFILIAL ) AND ( CDSERVFILI =:P_CDSERVFILI ) ) UNION ALL SELECT DISTINCT 'P' AS IDTIPO FROM RECEPADR LEFT JOIN PRATPADR ON PRATPADR.CDPRATO = RECEPADR.CDPRATO AND PRATPADR.CDFILAUXPRAT = RECEPADR.CDFILAUXPRAT WHERE ( RECEPADR.CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( RECEPADR.CDPRATO = P.CDPRATO ) AND (NVL(PRATPADR.IDATIVO,'S') = 'S') AND ( :P_IDUTRPADCA = 'S' ) AND NOT EXISTS (SELECT CDPRATO FROM RECEFILI WHERE ( CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( CDPRATO = P.CDPRATO ) AND ( CDFILIAL =:P_CDFILIAL ) ) AND NOT EXISTS (SELECT CDPRATO FROM RECESERV WHERE ( CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( CDPRATO = P.CDPRATO ) AND ( CDFILIAL =:P_CDFILIAL ) AND ( CDSERVFILI =:P_CDSERVFILI ) ) ) AS TIPORECEITA FROM PRATO P LEFT JOIN (SELECT GS.NRGRUPOPRATOSERV, G.NMGRUPOPRATO, IT.NRITGRUPOPRATOSERV, IT.CDPRATO, IT.CDFILAUXPRAT FROM GRUPOPRATO G INNER JOIN GRUPOPRATOSERV GS ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO AND G.CDFILIAL = :P_CDFILIAL INNER JOIN ITGRUPOPRATOSERV IT ON IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV ) G ON G.CDFILAUXPRAT = P.CDFILAUXPRAT AND G.CDPRATO = P.CDPRATO WHERE ( P.CDFILAUXPRAT = 'P' OR P.CDFILAUXPRAT =:P_CDFILIAL ) AND ( P.CDNVPRATO =:P_CDNVPRATO ) AND ( P.IDATIVO = 'S' ) ) WHERE TIPORECEITA IS NOT NULL AND NRGRUPOPRATOSERV IS NULL ORDER BY NMPRATO`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI},
            P_CDNVPRATO:        {val: params.CDNVPRATO},
            P_IDUTRPADCA:       {val: params.IDUTRPADCA},
            P_IDTIPORESTRICAO:  {val: params.IDTIPORESTRICAO}
        });
    },
    async get_sequencialCenarioOriginal ( params ) {
        const SQL = `SELECT NRSEQCNFCARD, NRSEQCNFCARDORIG FROM TIPOPRATCOMUMSERV WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI`;
        return this.execQuery( SQL, {
            P_CDFILIAL:         {val: params.CDFILIAL},
            P_CDSERVFILI:       {val: params.CDSERVFILI}
        });
    },
    async getJsonCenarioVinculo( NRCENARIOVINCULO ){
        if( !NRCENARIOVINCULO ) return [];
        const SQL = `SELECT NRCENARIOCARD, JSONCENARIO FROM CENARIOCARD WHERE NRCENARIOCARD = :P_NRCENARIOCARD`;
        return this.execQuery( SQL, {
            P_NRCENARIOCARD: NRCENARIOVINCULO
        });
    },
    addIncidencia_tp_gp_prato(dataFromOtm) {
        const _formatMes = (DTMESCARD) => {
            return moment(DTMESCARD, 'DD/MM/YYYY HH:mi:ss').format('MM/YY');
        };
        const _getPUAverageFromObject = (dataObject, lastMonthKey) => {
            const keysForAverage = Object.keys(dataObject).
            filter(k => k !== lastMonthKey && k.indexOf("pu") === 0)
            if (keysForAverage.length > 0) {
                const sum = keysForAverage.
                map(k => parseFloat(dataObject[k])).
                reduce((agg, c) => agg + c)

                return sum / keysForAverage.length
            } else {
                return null
            }
        }
        const _groupBy = function (xs, key) {
            return xs.reduce(function (rv, x) {
                (rv[x[key]] = rv[x[key]] || []).push(x);
                return rv;
            }, {});
        };
        const _weightedAverage = (a, weight, b) => {
            return (a * weight + b) / (weight + 1)
        };
        const {
            incidencia_tp_gp_prato
        } = dataFromOtm

        const incidencias = incidencia_tp_gp_prato.data.map(i => ({
            ...i,
            DTMESCARD: _formatMes(i.DTMESCARD)
        }))

        const meses = {}
        incidencias.forEach(i => {
            meses[i.DTMESCARD] = true
        })
        const monthsArray = Object.keys(meses)
        const lastMonth = monthsArray[monthsArray.length - 1]

        const incidenciasGroupedByPlateType = _groupBy(incidencias, 'CDTIPOPRAT');
        const incidenciasGroupedByPlateGroup = Object.values(incidenciasGroupedByPlateType).
        map(platesGroupedByPlateType => {
            return _groupBy(platesGroupedByPlateType, 'CDGRUPOPRATO')
        })


        const incidenciasGroupedByPlate = incidenciasGroupedByPlateGroup.
        map((plateGroups) => {
            const plateGroupsArray = Object.values(plateGroups);

            const firstPlateInType = plateGroupsArray[0][0]

            const plateTypeData = {
                ID: firstPlateInType.CDTIPOPRAT,
                pivo: firstPlateInType.CDTIPOPRAT,
                COD_TIP_PRATO: firstPlateInType.CDTIPOPRAT,
                NM: firstPlateInType.NMTIPOPRAT,
                isTPPRATO: true,
            }

            const gplist = plateGroupsArray.
            map(platesGroupedByPlateGroup => {

                const firstPlateData = platesGroupedByPlateGroup[0]
                const groupData = {
                    NM: firstPlateData.NMGRUPOPRATO,
                    pivo: firstPlateData.NRGRUPOPRATOSERV,
                    COD_TIP_PRATO: firstPlateData.CDTIPOPRAT
                }
                const platesInGroup = {}
                platesGroupedByPlateGroup.forEach(plate => {

                    const {
                        CDFILAUXPRAT,
                        CDPRATO
                    } = plate
                    const plateKey = CDFILAUXPRAT + CDPRATO
                    const previousPlateInfo = platesInGroup[plateKey] || {}
                    const currentMonth = plate.DTMESCARD
                    const platePU = parseFloat(plate.PU_PRATO)
                    const currentPUAverage = previousPlateInfo.mediaPU || 0

                    let monthsUsedInAverage = previousPlateInfo.monthsUsedInAverage || 0
                    let averagePU = currentPUAverage

                    if (currentMonth !== lastMonth) {
                        averagePU = _weightedAverage(currentPUAverage, monthsUsedInAverage, platePU)
                        monthsUsedInAverage = monthsUsedInAverage + 1
                    }
                    platesInGroup[plateKey] = {
                        ...previousPlateInfo,
                        ID: plateKey,
                        pivo: plateKey,
                        NM: plate.NMPRATO,
                        NRGRUPOPRATOSERV: plate.NRGRUPOPRATOSERV,
                        COD_TIP_PRATO: plate.CDTIPOPRAT,
                        ["qtde" + currentMonth]: plate.QTDE_PRATO,
                        ["cons" + currentMonth]: plate.CONSUMO_PRATO,
                        ["pu" + currentMonth]: platePU,
                        mediaPU: averagePU,
                        monthsUsedInAverage
                    }

                    groupData["qtde" + currentMonth] = plate.QTDE_GPPRATO
                    groupData["cons" + currentMonth] = plate.CONSUMO_GPPRATO
                    groupData["pu" + currentMonth] = parseFloat(plate.PU_GPPRATO)


                    plateTypeData["qtde" + currentMonth] = plate.QTDE_TPPRAT
                    plateTypeData["cons" + currentMonth] = plate.CONSUMO_TPPRAT
                    plateTypeData["pu" + currentMonth] = parseFloat(plate.PU_TPPRAT)

                })

                return {
                    ...groupData,
                    mediaPU: _getPUAverageFromObject(groupData, `pu${lastMonth}`),
                    plateList: Object.values(platesInGroup)
                }
            })
            return {
                ...plateTypeData,
                mediaPU: _getPUAverageFromObject(plateTypeData, `pu${lastMonth}`),
                gplist
            }
        })

        dataFromOtm.incidencia = {
            "incidenciaTabela": incidenciasGroupedByPlate,
            "meses": Object.keys(meses),
        }
        return dataFromOtm;
    },
    async retDadosCardInteligente( params ){
        const isValidParams = this.validateParameters( Object.keys(params) );
        if(!isValidParams) throw new Error('ERR_INVALID_PARAMETERS_ENDERR');
        const conn = {};
        const [
            configuracao_servico,
            frq_grupo_prato_mes,
            frq_opcoes_grupo_prato,
            frq_prato_mes,
            frq_prod_grupo_prato,
            grupo_prato,
            grupo_tipo_prato,
            limite_per_capita,
            prato_combinacao,
            prato_equivalente,
            receita_prato,
            res_grupo_prato,
            res_prato_combinacao,
            regras_flexiveis,
            tipo_prato,
            utilizacao_prato_semana,
            calendario,
            complexidade,
            prc_consumo_prato,
            seq_order,
            tp_corte,
            md_preparo,
            ing_princ,
            cong_days,
            lastWeekProds,
            conjunto_porcao,
            prato_sem_historico,
            restricao_de_repeticao_ingrediente,
            incidencia_tp_gp_prato,
            pratos_sem_grupo,
            sequencialCenarioOriginal
        ] = await Promise.all([
            this.getConfiguracaoServico( params ),
            this.getFrequenciaGrupoPratoMes( params ),
            this.getFrequenciaOpcoesGrupoPrato( params ),
            this.getFrequenciaPratosMes( params ),
            this.getFrequenciaProdutosGrupoPrato( params ),
            this.getGrupoPrato( params ),
            this.getGrupoTipoPrato( params ),
            this.getLimitePerCapita( params ),
            this.getPratoCombinacao( params ),
            this.getPratoEquivalente( params ),
            this.getReceitaPrato( params ),
            this.getRestricoesGrupoPrato( params ),
            this.getRestricoesPratoCombinacao( params ),
            this.getRestricoesFlexRules( params ),
            this.getTipoPrato( params ),
            this.getUtilizacaoPratoSemana( params ),
            this.getCalendario( params ),
            this.getComplexidade( params ),
            this.getConsumoPrato( params ),
            this.getSeqOrder( params ),
            this.getTpCorte( params ),
            this.getMdPreparo( params ),
            this.getIngPrinc ( params ),
            this.getCongDays ( params ),
            this.getLastWeekProds ( params ),
            this.getConjuntoPorcao( params ),
            this.getPrato_sem_historico ( params ),
            this.getPrato_restricao_de_repeticao_ingrediente ( params ),
            this.getIncidencia_tp_gp_prato ( params ),
            this.get_pratos_sem_grupo ( params ),
            this.get_sequencialCenarioOriginal ( params ),
        ]);
        BD.closeAll();
        return {
            configuracao_servico,
            frq_grupo_prato_mes,
            frq_opcoes_grupo_prato,
            frq_prato_mes,
            frq_prod_grupo_prato,
            grupo_prato,
            grupo_tipo_prato,
            limite_per_capita,
            prato_combinacao,
            prato_equivalente,
            receita_prato,
            res_grupo_prato,
            res_prato_combinacao,
            regras_flexiveis,
            tipo_prato,
            utilizacao_prato_semana,
            calendario,
            complexidade,
            prc_consumo_prato,
            seq_order,
            tp_corte,
            md_preparo,
            ing_princ,
            cong_days,
            lastWeekProds,
            conjunto_porcao,
            prato_sem_historico,
            restricao_de_repeticao_ingrediente,
            incidencia_tp_gp_prato,
            pratos_sem_grupo,
            sequencialCenarioOriginal
        };
    },
    async saveCenarioCard( params ){
        const {
            CDOPERADOR,
            JSONCENARIO,
            JSONCARDAPIO,
            VRPUCENARIO,
            NRCENARIOVINCULO
        } = params;

        const paramObj = {
            CDOPERADOR,
            JSONCENARIO,
            JSONCARDAPIO,
            VRPUCENARIO,
            DTCENARIOCARD  : moment().format('DD/MM/YYYY')
        };

        if( params.NRCENARIOCARD ){
            paramObj.NRCENARIOCARDORIGINAL = params.NRCENARIOCARD;
        }

        if( paramObj.NRCENARIOCARDORIGINAL ){
            const CENARIOCARD_SQL = `SELECT NRCENARIOCARD, NRCENARIOCARDORIGINAL FROM CENARIOCARD WHERE NRCENARIOCARD = :P_NRCENARIOCARD AND NRCENARIOCARDORIGINAL IS NOT NULL`;
            let [ cenData ] = this.execQuery(CENARIOCARD_SQL, {
                P_NRCENARIOCARD: params.NRCENARIOCARD
            });
            if( cenData ){
                paramObj.NRCENARIOCARD         = cenData.NRCENARIOCARD;
                paramObj.NRCENARIOCARDORIGINAL = cenData.NRCENARIOCARDORIGINAL;

                const UPDATE_CENARIOCARD_SQL = `UPDATE CENARIOCARD SET DTCENARIOCARD = :P_DTCENARIOCARD, CDOPERADOR = :P_CDOPERADOR, JSONCARDAPIO = :P_JSONCARDAPIO, VRPUCENARIO = :P_VRPUCENARIO, JSONCENARIO = :P_JSONCENARIO WHERE NRCENARIOCARD = :P_NRCENARIOCARD AND NRCENARIOCARDORIGINAL = :P_NRCENARIOCARDORIGINAL`;
                cenData = this.execQuery(UPDATE_CENARIOCARD_SQL, {
                    P_DTCENARIOCARD        : paramObj.DTCENARIOCARD,
                    P_CDOPERADOR           : paramObj.CDOPERADOR,
                    P_JSONCARDAPIO         : paramObj.JSONCARDAPIO,
                    P_VRPUCENARIO          : paramObj.VRPUCENARIO,
                    P_JSONCENARIO          : paramObj.JSONCENARIO,
                    P_NRCENARIOCARD        : paramObj.NRCENARIOCARD,
                    P_NRCENARIOCARDORIGINAL: paramObj.NRCENARIOCARDORIGINAL
                });
            }else{
                paramObj.CDFILIAL        = params.CDFILIAL;
                paramObj.CDSERVFILI      = params.CDSERVFILI;
                paramObj.DTMESCARD       = params.DTMESCARD;
                paramObj.QTCOMENSAIS     = params.QTCOMENSAIS;
                paramObj.IDTIPORESTRICAO = params.IDTIPORESTRICAO;
                paramObj.CDTIPOCUSTO     = params.NRTIPOCUSTO;
                paramObj.DTREFECUSTMES   = params.DTREFECUSTO;
                paramObj.VRCUSTPADR      = params.VRCUSTPADR;
                paramObj.NRORG           = params.NRORG;

                paramObj.NRCENARIOCARD = NovoCodigo.getCodeByTableName('CENARIOCARD', param.NRORG);
                // this.execSQL('CENARIOCARD', paramObj);

            }
        } else {
            paramObj.CDFILIAL         = params.CDFILIAL;
            paramObj.CDSERVFILI       = params.CDSERVFILI;
            paramObj.DTMESCARD        = params.DTMESCARD;
            paramObj.QTCOMENSAIS      = params.comensal_total;
            paramObj.IDTIPORESTRICAO  = params.IDTIPORESTRICAO;
            paramObj.CDTIPOCUSTO      = params.NRTIPOCUSTO;
            paramObj.DTREFECUSTMES    = params.DTREFECUSTO;
            paramObj.VRCUSTPADR       = params.VRCUSTPADR;
            paramObj.NRCENARIOVINCULO = params.NRCENARIOVINCULO;
            paramObj.NRORG            = params.NRORG;
            paramObj.VRPUCENARIO      = params.pu_aculm;
            paramObj.NRCENARIOCARD = await NovoCodigo.getCodeByTableName('CENARIOCARD', params.NRORG);
        }
        const parameters = Object.assign(
            BD.createParam('NRCENARIOCARD',  paramObj.NRCENARIOCARD),
            BD.createParam('DTCENARIOCARD',  paramObj.DTCENARIOCARD),
            BD.createParam('CDFILIAL',       paramObj.CDFILIAL),
            BD.createParam('CDSERVFILI',     paramObj.CDSERVFILI),

            BD.createParam('DTMESCARD',      paramObj.DTMESCARD),
            BD.createParam('CDOPERADOR',     paramObj.CDOPERADOR),
            BD.createParam('JSONCENARIO',    paramObj.JSONCENARIO),
            BD.createParam('JSONCARDAPIO',   paramObj.JSONCARDAPIO),

            BD.createParam('VRCUSTPADR',     parseFloat(paramObj.VRCUSTPADR || 0)),
            BD.createParam('VRPUCENARIO',    parseFloat(paramObj.VRPUCENARIO || 0)),
            BD.createParam('NRORG',          paramObj.NRORG),
            BD.createParam('IDATIVO',        'S'),

            BD.createParam('DTINCLUSAO',     moment().format('DD/MM/YYYY')),
            BD.createParam('DTULTATU',       null),
            BD.createParam('NRORGINCLUSAO',  paramObj.NRORG),
            BD.createParam('CDOPERINCLUSAO', paramObj.CDOPERADOR),

            BD.createParam('NRORGULTATU',     null),
            BD.createParam('CDOPERULTATU',    null),
            BD.createParam('IDTIPORESTRICAO', paramObj.IDTIPORESTRICAO),
            BD.createParam('QTCOMENSAIS',     paramObj.QTCOMENSAIS),

            BD.createParam('DTREFECUSTMES',         paramObj.DTREFECUSTMES),
            BD.createParam('CDTIPOCUSTO',           paramObj.CDTIPOCUSTO),
            BD.createParam('NRCENARIOCARDORIGINAL', paramObj.NRCENARIOCARDORIGINAL || null),
            BD.createParam('NRCENARIOVINCULO',      paramObj.NRCENARIOVINCULO || null)
        );
        const response = await BD.execSQL(`
                INSERT INTO CENARIOCARD
                    (
                        NRCENARIOCARD, DTCENARIOCARD, CDFILIAL,                 CDSERVFILI,
                        DTMESCARD,     CDOPERADOR,    JSONCENARIO,              JSONCARDAPIO,
                        VRCUSTPADR,    VRPUCENARIO,   NRORG,                    IDATIVO,
                        DTINCLUSAO,    DTULTATU,      NRORGINCLUSAO,            CDOPERINCLUSAO,
                        NRORGULTATU,   CDOPERULTATU,  IDTIPORESTRICAO,          QTCOMENSAIS,
                        DTREFECUSTMES, CDTIPOCUSTO,   NRCENARIOCARDORIGINAL,    NRCENARIOVINCULO
                    )
                VALUES (
                        :NRCENARIOCARD, :DTCENARIOCARD, :CDFILIAL,              :CDSERVFILI,
                        :DTMESCARD,     :CDOPERADOR,    :JSONCENARIO,           :JSONCARDAPIO,
                        :VRCUSTPADR,    :VRPUCENARIO,   :NRORG,                 :IDATIVO,
                        :DTINCLUSAO,    :DTULTATU,      :NRORGINCLUSAO,         :CDOPERINCLUSAO,
                        :NRORGULTATU,   :CDOPERULTATU,  :IDTIPORESTRICAO,       :QTCOMENSAIS,
                        :DTREFECUSTMES, :CDTIPOCUSTO,   :NRCENARIOCARDORIGINAL, :NRCENARIOVINCULO
                )
                `, parameters);
        console.log(`CENARIO: ${paramObj.NRCENARIOCARD}`);
        return {"NRCENARIOCARD": paramObj.NRCENARIOCARD};

    },
    async updateItGeracaoCard( NRCENARIOCARD, NRITGERACAOCARD ){
        const SQL_UPDATE_ITGERACAOCARD = `UPDATE ITGERACAOCARD SET NRCENARIOCARD = :P_NRCENARIOCARD WHERE NRITGERACAOCARD = :P_NRITGERACAOCARD`;
        const parameters = Object.assign(
            BD.createParam('P_NRCENARIOCARD', NRCENARIOCARD),
            BD.createParam('P_NRITGERACAOCARD', NRITGERACAOCARD),
        );
        const resp = await BD.execSQL(SQL_UPDATE_ITGERACAOCARD, parameters);
        return resp;
    },
    async insereCenario( params ){
        try{
            const route = 'CENARIO';
            const { $token } = params;
            const perm = PermissionManager.userHasPermissionTo("delete", route, $token);
            if(!perm){
                throw new BackOfficeException("Usuário não tem permissão para realizar essa operação.");
            }
            params.PARAMS = params;
            const retorno = await this.saveCenarioCard(params);
            if(retorno && retorno.NRCENARIOCARD) {
                const resp = await this.updateItGeracaoCard( retorno.NRCENARIOCARD, params.NRITGERACAOCARD );
            }
            return retorno;
        } catch ( error ){
            console.log(error);
        }
    },
    async insereLog( params ){
        const SQL = `
            INSERT INTO LOGITGERACAOCARD
            (
                NRLOGITGERACAOCARD, NRITGERACAOCARD, IDTIPOLOG,   DSLOGGERACAOCARD,
                IDATIVO,            DTINCLUSAO,      DTULTATU,    CDOPERINCLUSAO,
                NRORGULTATU,        CDOPERULTATU,    DSLOGINTERNO
            )
            VALUES
            (
                :NRLOGITGERACAOCARD, :NRITGERACAOCARD, :IDTIPOLOG,   :DSLOGGERACAOCARD,
                :IDATIVO,            :DTINCLUSAO,      :DTULTATU,    :CDOPERINCLUSAO,
                :NRORGULTATU,        :CDOPERULTATU,    :DSLOGINTERNO
            )
        `;
        const NRLOGITGERACAOCARD = await NovoCodigo.getCodeByTableName('LOGITGERACAOCARD', params.NRORG);
        const parameters = Object.assign(
            BD.createParam('NRLOGITGERACAOCARD', NRLOGITGERACAOCARD),
            BD.createParam('NRITGERACAOCARD',    params.NRITGERACAOCARD),
            BD.createParam('IDTIPOLOG',          params.IDTIPOLOG),
            BD.createParam('DSLOGGERACAOCARD',   params.DSLOGGERACAOCARD || " "),

            BD.createParam('IDATIVO',            'S' ),
            BD.createParam('DTINCLUSAO',         moment().format('DD/MM/YYYY')),
            BD.createParam('DTULTATU',           null),

            BD.createParam('CDOPERINCLUSAO',     params.CDOPERADOR),
            BD.createParam('NRORGULTATU',        null),
            BD.createParam('CDOPERULTATU',       null),

            BD.createParam('DSLOGINTERNO',       params.DSLOGINTERNO || null)
        );
        const result = await BD.execSQL(SQL, parameters);
        return result;
    },
    async deleteOldLog( NRITGERACAOCARD ){
        const SQL = `DELETE LOGITGERACAOCARD WHERE NRITGERACAOCARD = :P_NRITGERACAOCARD`;
        const result = await BD.execSQL(SQL, BD.createParam('P_NRITGERACAOCARD', NRITGERACAOCARD));
        return result;
    },
    getFirstRecordObject( resultSQL ){
        if(!resultSQL || !resultSQL.data) return {};
        let dataSet = resultSQL.data;
        if(Array.isArray(dataSet) && dataSet.length){
            let [ FIRST_RECORD ] = dataSet;
            return FIRST_RECORD;
        }
        return {};
    },
    async getBackOfficeData( NRGERACAOCARD, NRITGERACAOCARD ){
        const SQL_IT_GERACAOCARD = `SELECT GC.DTMESGERACAOCARD AS DTMESCARD, IC.CDFILIAL, FI.NMFILIAL, IC.CDSERVFILI, SE.NMSERVICO, IC.DTREFERENCIA AS DTREFECUSTO, IC.NRCENARIOVINCULO, IC.VRCUSTOPADRAO AS VRCUSTPADR, IC.VRCUSTOMAX AS VLR_CUSTO_MAX, IC.VRCUSTOMIN AS VLR_CUSTO_MIN, IC.CDTIPOCUSTO AS NRTIPOCUSTO, GC.IDTIPORESTRICAO, IC.CDGRUPPROD AS CDGRUPPRODOTMCARD, IC.IDPERMITECONGELAR, IC.QTDIASCONG FROM GERACAOCARD GC INNER JOIN ITGERACAOCARD IC ON GC.NRGERACAOCARD = IC.NRGERACAOCARD INNER JOIN FILIAL FI ON FI.CDFILIAL = IC.CDFILIAL INNER JOIN SERVICO SE ON SE.CDFILIAL = IC.CDFILIAL AND SE.CDSERVFILI = IC.CDSERVFILI WHERE GC.NRGERACAOCARD = :P_NRGERACAOCARD AND IC.NRITGERACAOCARD = :P_NRITGERACAOCARD`;
        const IT_GERACAOCARD = await this.execQuery(SQL_IT_GERACAOCARD, {
            "P_NRGERACAOCARD": NRGERACAOCARD,
            "P_NRITGERACAOCARD": NRITGERACAOCARD
        });
        const resultItGeracaoCard = this.getFirstRecordObject(IT_GERACAOCARD);
        const { CDFILIAL, CDSERVFILI, DTMESCARD, NRCENARIOVINCULO, IDTIPORESTRICAO } = resultItGeracaoCard;
        const SQL_PARAMGERAL = `SELECT CDPICTPRAT, CDPICTPROD, DSDNSCARDOTIMIZADO, IDUTRPADCA FROM PARAMGERAL`;
        const PARAMGERAL = await this.execQuery(SQL_PARAMGERAL, {});
        const resultParamGeral = this.getFirstRecordObject(PARAMGERAL);

        const SQL_COST_TYPE = `SELECT CDTIPOCUSTO, NMTIPOCUSTO FROM ( SELECT CDTIPOCUSTO, NMTIPOCUSTO, CASE WHEN CDTIPOCUSTOPLA IS NOT NULL THEN 0 ELSE 1 END AS ORDEM FROM TIPOCUSTO TP LEFT JOIN PARAMPLANEJA PJ ON TP.CDTIPOCUSTO = PJ.CDTIPOCUSTOPLA AND CDFILIAL = :CDFILIAL WHERE IDUTTPCUSTO = 'S' AND ( UPPER(NMTIPOCUSTO) LIKE '%' || UPPER(:NMTIPOCUSTO) || '%' OR :NMTIPOCUSTO IS NULL ) ) TP ORDER BY ORDEM, CDTIPOCUSTO`;
        const COST_TYPE = await this.execQuery(SQL_COST_TYPE, {
            CDFILIAL,
            NMTIPOCUSTO: null
        });
        const resultCostType = this.getFirstRecordObject(COST_TYPE);

        const SQL_GROUPPROD = `SELECT CDGRUPPROD, NMGRUPPROD FROM GRUPPROD ORDER BY CDGRUPPROD`;
        const GROUPPROD = await this.execQuery(SQL_GROUPPROD, {});
        const resultGroupProd = this.getFirstRecordObject(GROUPPROD);
        let CENARIOCARDVINCULO = null;
        if(NRCENARIOVINCULO) {
            CENARIOCARDVINCULO = await this.getJsonCenarioVinculo(  NRCENARIOVINCULO );
        }
        const { JSONCENARIO } = this.getFirstRecordObject(CENARIOCARDVINCULO);
        let rowResult = {
            costType: resultCostType,
            groupProdLimit: resultGroupProd,
        };
        if(JSONCENARIO){
            let parsedCardapio = JSON.parse(JSONCENARIO).cardapio;
            parsedCardapio.datas.forEach((d, dkey)  => {
              d.pratos.forEach((p, pkey) => {
                parsedCardapio.datas[dkey].pratos[pkey] = {codprato: p.codprato, sequencial: p.sequencial};
              });
              parsedCardapio.datas[dkey] = {dia: d.dia, pratos: d.pratos};
            });
            rowResult.parsedCardapio = parsedCardapio;
            rowResult.NRCENARIOVINCULO = NRCENARIOVINCULO;
        }

        Object.assign(
            rowResult,
            resultItGeracaoCard,
            resultParamGeral
        );
        if(rowResult.VLR_CUSTO_MAX) {
            rowResult.VLR_CUSTO_MAX = parseFloat(rowResult.VLR_CUSTO_MAX.toFixed(5));
        }
        if(rowResult.VLR_CUSTO_MIN) {
            rowResult.VLR_CUSTO_MIN = parseFloat(rowResult.VLR_CUSTO_MIN.toFixed(5));
        }
        if(rowResult.VRCUSTPADR) {
            rowResult.VRCUSTPADR = parseFloat(rowResult.VRCUSTPADR.toFixed(5));
        }
        return rowResult;
    }
}

export default CardapioInteligente;
